#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests that hit the network.

Those tests are in a separate file to allow the base set of tests to
be ran without internet access.
"""

import unittest

import os
import sys
sys.path.insert(0, os.path.dirname(__file__) + '/../..')

from monkeysign.gpg import TempKeyring
from test_lib import TestTimeLimit, AlarmException, find_test_file, skipUnlessNetwork


@skipUnlessNetwork()
class TestGpgNetwork(TestTimeLimit):

    """Separate test cases for functions that hit the network

each test needs to run under a specific timeout so we don't wait on
the network forever"""

    def setUp(self):
        self.gpg = TempKeyring()
        self.gpg.context.set_option('keyserver', 'pool.sks-keyservers.net')
        TestTimeLimit.setUp(self)

    def test_fetch_keys(self):
        """test key fetching from keyservers"""
        try:
            self.assertTrue(self.gpg.fetch_keys('4023702F'))
        except AlarmException:
            raise unittest.case._ExpectedFailure(sys.exc_info())

    def test_special_key(self):
        """test a key that sign_key had trouble with"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))
        try:
            self.assertTrue(self.gpg.fetch_keys('3CCDBB7355D1758F549354D20B123309D3366755'))
        except AlarmException:
            raise unittest.case._ExpectedFailure(sys.exc_info())
        self.assertTrue(self.gpg.sign_key('3CCDBB7355D1758F549354D20B123309D3366755', True))

    def tearDown(self):
        TestTimeLimit.tearDown(self)
        del self.gpg


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals

__description__ = 'OpenPGP key exchange for humans'
__long_description__ = """
monkeysign is a tool to overhaul the OpenPGP keysigning experience and
bring it closer to something that most primates can understand.

The project makes use of cheap digital cameras and the type of bar
code known as a QRcode to provide a human-friendly yet still-secure
keysigning experience.

No more reciting tedious strings of hexadecimal characters.  And, you
can build a little rogue's gallery of the people that you have met and
exchanged keys with!
"""
try:
    from _version import version
except ImportError:
    try:
        from setuptools_scm import get_version
        version = get_version()
    except (ImportError, LookupError):
        version = '???'
__version__ = version
__copyright__ = """Copyright (C) 2010-2013  Antoine Beaupré, Jerome Charaoui
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
For details see the COPYRIGHT file distributed along this program."""

__license__ = """
    This package is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    This package is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this package; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
"""

__website__ = 'http://web.monkeysphere.info/monkeysign'
__documentation__ = 'http://monkeysign.readthedocs.io/en/' + __version__
# hack to include the credits in the documentation
# maybe this should be parsed from a AUTHORS file instead, see
# https://0xacab.org/monkeysphere/monkeysign/issues/56
# if you feel comptent to respond to Code of Conduct reports,
# make sure you add your email address below
# credits-start
__authors__ = ['In alphabetical order:',
               '',
               'Antoine Beaupré',
               'Daniel Kahn Gillmor',
               'Gabriel Fillion',
               'Jérôme Charaoui',
               'Kristian Fiskerstrand',
               'Philip Jägenstedt',
               'Ramakrishnan Muthukrishnan',
               'Simon Fondrie-Teitler',
               'Tobias Mueller',
               ]
__documenters__ = ['In alphabetical order:',
                   '',
                   'Antoine Beaupré',
                   'Emma Peel',
                   ]
__translators__ = ['In alphabetical order:',
                   '',
                   'Antoine Beaupré',
                   'Ahmed El Azzabi',
                   'Gonzalo Exequiel Pedone',
                   'Michael R. Lawrence',
                   'Michal Čihař',
                   ]



# -*- coding: utf-8 -*-
#
#    Copyright (C) 2010 Jerome Charaoui
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import copy
from glob import glob
import os
import pkg_resources
import re
import StringIO
import stat
import subprocess  # nosec
import webbrowser

from glib import GError
import gobject
import gtk
import PIL
import pygtk
pygtk.require('2.0')
import zbar
import zbarpygtk

from qrencode import encode_scaled as _qrencode_scaled

from monkeysign.gpg import Keyring, GpgRuntimeError
from monkeysign.ui import MonkeysignUi
import monkeysign.translation
from monkeysign.msg_exception import errorhandler
import monkeysign


class MonkeysignScanUi(MonkeysignUi):
    """sign a key in a safe fashion using a webcam to scan for qr-codes

This command will fire up a graphical interface and turn on the webcam
(if available) on this computer. It will also display a qr-code of
your main OpenPGP key.

The webcam is used to capture an OpenPGP fingerprint represented as a
qrcode (or whatever the zbar library can parse) and then go through a
signing process.

The signature is then encrypted and mailed to the user. This leave the
choice of publishing the certification to that person and makes sure
that person owns the identity signed.

This program assumes you have gpg-agent configure to prompt for
passwords.
"""

    @classmethod
    def sysinfo(cls):
        info = MonkeysignUi.sysinfo()
        info.append('GTK: %s.%s.%s' % gtk.gtk_version)
        info.append('PyGTK: %s.%s.%s' % gtk.pygtk_version)
        zbar_version_str = ".".join([str(x) for x in zbar.version()])
        info.append('ZBar: %s' % zbar_version_str)
        info.append('PIL: %s' % PIL.VERSION)
        try:
            qrencode_version = pkg_resources.get_distribution('qrencode').version
        except pkg_resources.DistributionNotFound:
            qrencode_version = '?'
        info.append('qrencode: %s' % qrencode_version)
        return info

    @errorhandler
    def main(self):
        # threads *must* be properly initialized to use zbarpygtk
        gtk.gdk.threads_init()
        gtk.gdk.threads_enter()

        self.window = MonkeysignScan()
        self.window.msui = self

        # XXX: this probably belongs lower in the stack,
        # because we don't want to create a temporary keyring
        # just when we start the graphical UI, but instead
        # really when we sign
        MonkeysignUi.main(self)

        gtk.main()
        gtk.gdk.threads_leave()

    def yes_no(self, prompt, default=None):
        """we ignore default! gotta fix that"""
        md = gtk.MessageDialog(self.window,
                               gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_QUESTION,
                               gtk.BUTTONS_YES_NO, prompt)
        with gtk.gdk.lock:
            response = md.run()
        md.destroy()
        return response == gtk.RESPONSE_YES

    def acknowledge(self, prompt):
        """untested"""
        prompt = _('Press OK when %s') % prompt
        md = gtk.MessageDialog(self.window,
                               gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_QUESTION,
                               gtk.BUTTONS_OK, prompt)
        with gtk.gdk.lock:
            md.run()
        md.destroy()

    def abort(self, prompt):
        """we don't actually abort, just exit threads and resume capture"""
        md = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, prompt)
        with gtk.gdk.lock:
            md.run()
        md.destroy()
        self.window.resume_capture()

    def warn(self, prompt):
        """display the message but let things go"""
        md = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, prompt)
        with gtk.gdk.lock:
            md.run()
        md.destroy()

    def choose_uid(self, prompt, key):
        md = gtk.Dialog(prompt, self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK,
                         gtk.RESPONSE_ACCEPT))
        # simple explanation
        l = gtk.Label(prompt)
        md.vbox.pack_start(l)
        l.show()
        # list of uids
        self.uid_radios = None
        for uid in key.uidslist:
            r = gtk.RadioButton(self.uid_radios, uid.uid)
            r.show()
            md.vbox.pack_start(r)
            if self.uid_radios is None:
                self.uid_radios = r
                self.uid_radios.set_active(True)

        with gtk.gdk.lock:
            response = md.run()

        label = None
        if response == gtk.RESPONSE_ACCEPT:
            self.log(_('okay, signing'))
            label = [radio for radio in self.uid_radios.get_group()
                     if radio.get_active()][0].get_label()
        else:
            self.log(_('user denied signature'))
        md.destroy()
        return label


class MonkeysignScan(gtk.Window):
    ui = '''<ui>
    <menubar name="menu">
        <menu action="file">
            <menuitem action="open"/>
            <menuitem action="save"/>
            <menuitem action="print"/>
            <separator name="FileSeparator2"/>
            <menuitem action="quit"/>
        </menu>
        <menu action="edit">
            <menuitem action="copy"/>
            <menuitem action="preferences"/>
        </menu>
        <menu action="identity"/>
        <menu action="video"/>
        <menu action="help">
            <menuitem action="about"/>
            <menuitem action="documentation"/>
        </menu>
    </menubar>
    </ui>'''

    def __init__(self):
        super(MonkeysignScan, self).__init__()

        # modal dialogs to destroy
        self.md = []

        # Set up main window
        self.set_title(_('Monkeysign (scan)'))
        self.set_position(gtk.WIN_POS_CENTER)
        self.connect("destroy", self.destroy)

        self.set_icon()
        self.create_menu()
        video_found = self.create_video_controls()
        self.create_webcam_display(video_found)
        self.create_qrcode_display()
        self.create_secret_keys_display()
        self.last_allocation = self.get_allocation()

        # top horizontal box: webcam and qrcode
        mainhbox = gtk.HBox()
        mainhbox.pack_start(self.zbarwidget, False, False, 10)
        mainhbox.pack_start(self.qrcodewidget, True, True, 10)

        # main vertical box: the above with a menu on top
        mainvbox = gtk.VBox()
        mainvbox.pack_start(self.uimanager.get_widget('/menu'), False, False)
        mainvbox.pack_start(mainhbox, False, False, 10)
        self.add(mainvbox)

        # Start the show
        self.show_all()

    def set_icon(self):
        # setup icon
        try:
            icon = pkg_resources.resource_filename(pkg_resources.Requirement.parse("monkeysign"), 'monkeysign.svg')
            gtk.window_set_default_icon_from_file(icon)
        except GError:
            # misconfigured, ignore
            # XXX: no access to logging, again
            # self.msui.warn("could not find icon: %s" % e)
            pass
        except pkg_resources.DistributionNotFound:
            # not installed system-wide, ignore
            pass
        finally:
            # last resort, try the FHS and source directories
            for icon in ["/usr/share/icons/hicolor/scalable/apps/monkeysign.svg",
                         "/usr/share/pixmaps/monkeysign.png",
                         os.path.dirname(__file__) +
                         "/../doc/ui-mockups/logo.simple.svg"]:
                if os.path.exists(icon):
                    gtk.window_set_default_icon_from_file(icon)

    def create_menu(self):
        """create the main menu"""
        self.uimanager = gtk.UIManager()
        accelgroup = self.uimanager.get_accel_group()
        self.add_accel_group(accelgroup)
        self.actiongroup = gtk.ActionGroup('MonkeysignGen_Menu')
        self.actiongroup.add_actions([
                ('file', None, _('_File')),
                ('open', gtk.STOCK_OPEN, _('_Open image...'), None, None, self.import_image),
                ('save', gtk.STOCK_SAVE, _('_Save QR code as...'), None, None, self.save_qrcode),
                ('print', gtk.STOCK_PRINT, _('_Print QR code...'), None, None, self.print_op),
                ('edit', None, '_Edit'),
                ('copy', gtk.STOCK_COPY, _('_Copy QR code'), None, _('Copy image to clipboard'), self.clip_qrcode),
                ('preferences', gtk.STOCK_PREFERENCES, _('P_references'), None, _('Edit Monkeysign preferences'), self.edit_preferences),
                ('identity', None, _('Identity'), None, _('Choose identity')),
                ('video', None, _('Video device'), None, _('Select video device to use')),
                ('quit', gtk.STOCK_QUIT, _('_Quit'), None, None, self.destroy),
                ('help', None, _('_Help')),
                ('about', gtk.STOCK_ABOUT, _('_About'), None, None, self.about_dialog),
                ('documentation', gtk.STOCK_HELP, _('Documentation'), None, None, self.documentation_dialog),
                ])
        self.uimanager.insert_action_group(self.actiongroup, 0)
        self.uimanager.add_ui_from_string(self.ui)
        self.uimanager.get_widget('/menu/help').set_right_justified(True)

    def create_video_controls(self):
        """create controls to choose the video device"""
        i = 0
        video = False
        radiogroup = self.add_video_device('disable', _('Disable video'), None, i)
        i += 1
        for path in glob("/dev/video[0-9]*"):
                if not os.access(path, os.F_OK):
                    continue
                info = os.stat(path)
                if stat.S_ISCHR(info.st_mode) and os.major(info.st_rdev) == 81:
                    try:
                        name = open('/sys/class/video4linux/%s/name'
                                    % os.path.basename(path)).read()
                        label = "%s (%s)" % (name, path)
                    except IOError:
                        label = path
                        pass
                    self.add_video_device(path, label,
                                          path, i).set_group(radiogroup)
                    video = path
                    i += 1
        radiogroup.set_current_value(i-1)
        return video

    def add_video_device(self, name, label, path, i):
        """helper function to add an entry for a video device"""
        self.uimanager.add_ui(self.uimanager.new_merge_id(),
                              '/menu/video', label, name,
                              gtk.UI_MANAGER_AUTO, True)
        action = gtk.RadioAction(name, label, label, None, i)
        action.connect('activate', self.video_changed, path)
        self.actiongroup.add_action(action)
        return action

    def create_webcam_display(self, video_found):
        """create the webcam preview widgets"""
        if video_found:
            self.zbar = zbarpygtk.Gtk()
            self.zbar.connect("decoded-text", self.decoded)
            self.zbar.set_video_device(video_found)
            self.zbarframe = gtk.Frame()
            self.zbarframe.add(self.zbar)
            # set a normal window size for the webcam, for larger resolutions
            # see: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=723154
            # 640x480 is mostly arbitrary, based on an old VGA standard
            self.zbar.set_size_request(640, 480)
        else:
            camframe = gtk.Frame()
            self.zbarframe = camframe
            error_icon = gtk.Image()
            error_icon.set_from_stock(gtk.STOCK_DIALOG_ERROR,
                                      gtk.ICON_SIZE_DIALOG)
            vbox = gtk.VBox()
            error_icon_bottom = gtk.Alignment(0, 1, 1, 0)
            error_icon_bottom.add(error_icon)
            error_label_top = gtk.Alignment(0, 0, 1, 0)
            error_label_top.add(gtk.Label(_('No video device detected.')))
            vbox.pack_start(error_icon_bottom)
            vbox.pack_start(error_label_top)
            vbox.set_size_request(320, 320)
            camframe.add(vbox)
        self.zbarwidget = gtk.VBox()
        label = gtk.Label(_('This is the output of your webcam, align a qrcode in the image to scan a fingerprint.'))
        label.set_line_wrap(True)
        self.zbarwidget.pack_start(label, False)
        self.zbarwidget.pack_start(self.zbarframe)

    def create_qrcode_display(self):
        """create the QR code display"""
        # Hold QR code in pixbuf
        self.pixbuf = None
        # Remember last allocation when resizing
        self.last_allocation = gtk.gdk.Rectangle()
        # Initialise print settings
        self.printsettings = None
        # hook up to resize events
        self.connect("expose-event", self.expose_event)

        self.qrcode = gtk.Image()  # QR Code widget
        self.clip = gtk.Clipboard()  # Clipboard
        self.qrcodewidget = gtk.VBox()
        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
        swin.add_with_viewport(self.qrcode)
        label = gtk.Label(_('This is a QR-code version of your OpenPGP fingerprint. Scan this with another Monkeysign to transfer your fingerprint.'))
        label.set_line_wrap(True)
        self.qrcodewidget.pack_start(label, False)
        self.qrcodewidget.pack_start(swin)

        self.qrcodewidget.connect("event", self.qr_code_event)

    def create_secret_keys_display(self):
        """list the secret keys for selection somewhere"""
        i = 0
        radiogroup = None
        for fpr, key in Keyring().get_keys(None, True, False).items():
            if key.invalid or key.disabled \
               or key.expired or key.revoked:
                continue
            # weird key without uids, skip (see BTS#723152)
            if not key.uidslist:
                continue
            uid = key.uidslist[0].uid
            self.uimanager.add_ui(self.uimanager.new_merge_id(),
                                  '/menu/identity', fpr, fpr,
                                  gtk.UI_MANAGER_AUTO, True)
            action = gtk.RadioAction(fpr, "%s (%s)"
                                     % (uid, key.keyid()),
                                     str(key), None, i)
            i += 1
            action.connect('activate', self.uid_changed, key)
            if radiogroup is None:
                radiogroup = action
            else:
                action.set_group(radiogroup)
            self.actiongroup.add_action(action)
        self.uimanager.add_ui(self.uimanager.new_merge_id(), '/menu/identity', _('Hide QR code'), 'hide', gtk.UI_MANAGER_AUTO, True)
        action = gtk.RadioAction('hide', _('Hide QR code'), _('Hide QR code'), None, i)
        action.connect('activate', self.uid_changed, None)
        if radiogroup is None:
            radiogroup = action
        else:
            action.set_group(radiogroup)
        self.actiongroup.add_action(action)
        # fire off activation hook once
        radiogroup.set_current_value(0)

    def expose_event(self, widget, event):
        """When window is resized, regenerate the QR code"""
        if self.get_allocation() != self.last_allocation:
            self.last_allocation = self.get_allocation()
            self.draw_qrcode()

    def uid_changed(self, action, key):
        """refresh the qrcode when the selected key changes"""
        if action.get_active():
            self.active_key = key
            self.draw_qrcode()

    def draw_qrcode(self):
        """draw the qrcode from the key fingerprint"""
        if self.active_key:
            self.pixbuf = self.image_to_pixbuf(self.make_qrcode(self.active_key.fpr))
            self.qrcode.set_from_pixbuf(self.pixbuf)
        else:
            self.qrcode.set_from_stock(gtk.STOCK_DIALOG_ERROR,
                                       gtk.ICON_SIZE_DIALOG)

    def video_changed(self, action, path):
        """callback invoked when a new video device is selected from the
        drop-down list.  sets the new device for the zbar widget,
        which will eventually cause it to be opened and enabled
        """
        try:
            if action.get_active():
                self.zbar.set_video_enabled(path is not None)
                if path is not None:
                    self.zbar.set_video_device(path)
        except AttributeError:  # nosec
            pass

    def make_qrcode(self, fingerprint):
        """Given a fingerprint, generate a QR code image with appropriate
        prefix"""
        rect = self.qrcodewidget.get_allocation()
        if rect.width < rect.height:
            size = rect.width - 15
        else:
            size = rect.height - 15
        version, width, image = _qrencode_scaled('OPENPGP4FPR:' +
                                                 fingerprint,
                                                 size, 0, 1, 2, True)
        return image

    def import_image(self, widget):
        """Use a file chooser dialog to import an image containing a QR
        code"""

        self.dialog = gtk.FileChooserDialog(_("Open QR code image"), None, gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        self.dialog.set_default_response(gtk.RESPONSE_OK)
        response = self.dialog.run()
        filename = self.dialog.get_filename()
        self.dialog.destroy()
        if response == gtk.RESPONSE_OK:
            # XXX: without this, the ask() method later freeze, go figure
            gtk.gdk.threads_leave()
            try:
                verified = False
                for suffix in ['.asc', '.sig']:
                    if os.path.exists(filename + suffix):
                        # armored signature exists, verify it
                        verified = self.msui.keyring.verify_file(filename
                                                                 + suffix,
                                                                 filename)
                if not verified:
                    raise GpgRuntimeError(0, _('cannot find signature for image file'))
            except GpgRuntimeError :
                self.msui.warn(_("The image provided cannot be verified using a trusted OpenPGP signature.\n\nMake sure the image comes from a trusted source (e.g. your own camera, which you have never left unsurveilled) before signing this!\n\nDO NOT SIGN UNTRUSTED FINGERPRINTS!\n\nTo get rid of this warning, if you really trust this image, use the following command to sign the file\n\n    gpg -s --detach %s\n") % filename)
            else:
                self.msui.log(_('image signature verified successfully'))
            self.scan_image(filename)
        return

    def scan_image(self, filename):
        """Scan an image for QR codes"""

        self.capture = gtk.Image()
        self.capture.set_from_file(filename)
        self.capture.show()

        # create a reader
        scanner = zbar.ImageScanner()

        # configure the reader
        scanner.parse_config('enable')

        # obtain image data
        image = PIL.Image.open(filename)
        pil = image.convert('L')
        width, height = pil.size
        raw = pil.tostring()

        # wrap image data
        rawimage = zbar.Image(width, height, 'Y800', raw)

        # scan the image for barcodes
        scanner.scan(rawimage)

        # extract results
        found = False
        for symbol in rawimage:
            try:
                self.zbarframe.remove(self.zbar)
            except AttributeError:  # nosec
                # no video display, ignore
                pass
            self.zbarframe.add(self.capture)
            self.zbarframe.set_shadow_type(gtk.SHADOW_ETCHED_IN)
            self.process_scan(symbol.data)
            found = True
        if not found:
            self.msui.warn(_('no data found in image!'))

    def save_qrcode(self, widget=None):
        """Use a file chooser dialog to enable user to save the current QR
        code as a PNG image file"""
        if self.active_key is None:
            # XXX: without this, warn() freezes, go figure
            gtk.gdk.threads_leave()
            self.msui.warn(_('No identity selected. Select one from the identiy menu or generate a OpenPGP key if none is available.'))
            return
        key = self.active_key
        image = self.make_qrcode(key.fpr)
        dialog = gtk.FileChooserDialog(_('Save QR code'), None, gtk.FILE_CHOOSER_ACTION_SAVE, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_name(key.keyid() + '.png')
        dialog.show()
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
                name = dialog.get_filename()
                image.save(name, 'PNG')
        elif response == gtk.RESPONSE_CANCEL:
                pass
        dialog.destroy()
        return

    def clip_qrcode(self, widget=None):
        """copy the qrcode to the clipboard"""
        self.clip.set_image(self.pixbuf)

    def print_op(self, widget=None):
        """handler for the print QR code menu"""
        if self.active_key is None:
            # XXX: without this, warn() freezes, go figure
            gtk.gdk.threads_leave()
            self.msui.warn(_('No identity selected. Select one from the identiy menu or generate a OpenPGP key if none is available.'))
            return
        keyid = self.active_key.subkeys[0].keyid()
        print_op = gtk.PrintOperation()
        print_op.set_job_name('Monkeysign-'+keyid)
        print_op.set_n_pages(1)
        print_op.connect("draw_page", self.print_qrcode)
        res = print_op.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG,
                           self)

    def edit_preferences(self, operation=None):
        dialog = gtk.Dialog(_('Monkeysign preferences'), self,
                            gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dialog.set_default_response(gtk.RESPONSE_ACCEPT)
        dialog.show()
        content = dialog.get_content_area()
        args = copy.deepcopy(self.msui.options)
        for action in self.msui.parser._actions:
            if action.dest in self.msui.parser.ephemeral:
                continue
            val = vars(args)[action.dest]
            if isinstance(action, argparse._StoreAction):
                def edit_option(button, name):
                    vars(args)[name] = button.get_text()
                    print('%s is now %s' % (name, vars(args)[name]))
                box = gtk.HBox()
                label = gtk.Label(action.dest)
                box.pack_start(label)
                label.show()
                entry = gtk.Entry()
                if val is not None and val != action.default:
                    entry.set_text(val)
                entry.connect('changed', edit_option, action.dest)
                box.pack_start(entry)
                box.show()
                content.pack_end(box)
                entry.show()
            else:
                def toggle_option(button, name):
                    vars(args)[name] = button.get_active()
                    print('%s is now %s' % (name, vars(args)[name]))
                checkbox = gtk.CheckButton(action.dest)
                checkbox.set_tooltip_text(action.help)
                checkbox.set_active(val)
                checkbox.connect('toggled', toggle_option, action.dest)
                content.pack_end(checkbox)
                checkbox.show()
        response = dialog.run()
        if response == gtk.RESPONSE_ACCEPT:
            print('saving config')
            self.msui.parser.save_config(args)
            self.msui.options = args
        elif response == gtk.RESPONSE_CANCEL:
            pass
        dialog.destroy()
        return

    def print_qrcode(self, operation=None, context=None, page_nr=None):
        """actually print the qr code"""
        ctx = context.get_cairo_context()
        ctx.set_source_pixbuf(self.pixbuf, 0, 0)
        ctx.paint()
        ctx.restore()
        return

    def image_to_pixbuf(self, image):
        """Utility function to convert a PIL image instance to Pixbuf"""
        fd = StringIO.StringIO()
        image.save(fd, "ppm")
        contents = fd.getvalue()
        fd.close()
        loader = gtk.gdk.PixbufLoader("pnm")
        loader.write(contents, len(contents))
        pixbuf = loader.get_pixbuf()
        loader.close()
        return pixbuf

    def update_progress_callback(self, *args):
        """callback invoked for pulsating progressbar
        """
        if self.keep_pulsing:
            self.progressbar.pulse()
            return True
        else:
            return False

    def watch_out_callback(self, pid, condition):
        """callback invoked when gpg key download is finished
        """
        self.keep_pulsing = False
        try:
            self.dialog.destroy()
        except AttributeError:  # nosec
            # XXX: this should be handled better, bugfix for:
            # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=770900
            # this is actually because the key was
            # imported without having to create a dialog
            pass
        self.msui.log(_('fetching finished'))
        if condition == 0:
            # 2. copy the signing key secrets into the keyring
            self.msui.copy_secrets()
            # 3. for every user id (or all, if -a is specified)
            # 3.1. sign the uid, using gpg-agent
            self.msui.sign_key()

            # 3.2. export and encrypt the signature
            # 3.3. mail the key to the user
            self.msui.export_key()

            # 3.4. optionnally (-l), create a local signature and import in
            # local keyring
            # 4. trash the temporary keyring

            self.resume_capture()
            for md in self.md:
                md.destroy()
        return

    def decoded(self, zbar, data):
        """callback invoked when a barcode is decoded by the zbar widget.
        checks for an openpgp fingerprint
        """

        # Capture and display the video frame containing QR code
        self.zbarframe.set_shadow_type(gtk.SHADOW_NONE)
        alloc = self.zbarframe.allocation
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
                                True, 8, alloc.width, alloc.height)
        pixbuf.get_from_drawable(self.zbarframe.window,
                                 self.zbarframe.window.get_colormap(),
                                 alloc.x, alloc.y, 0, 0,
                                 alloc.width, alloc.height)
        self.capture = gtk.Image()
        self.capture.set_from_pixbuf(pixbuf)
        self.capture.show()
        self.zbarframe.remove(self.zbar)
        self.zbarframe.add(self.capture)
        self.zbarframe.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        # Disable video capture
        self.zbar.set_video_enabled(False)

        self.process_scan(data)

    def process_scan(self, data):
        """process zbar-scanned data"""

        self.msui.log(_('zbar captured a frame, looking for 40 character hexadecimal fingerprint in %s') % data)
        m = re.search("((?:[0-9A-F]{4}\s*){10})", data, re.IGNORECASE)

        if m is not None:
            # Found fingerprint, get it and strip spaces for GPG
            # XXX: not sure why passing it into msui is necessary
            self.msui.pattern = m.group(1).replace(' ', '')
            # 1. fetch the key into a temporary keyring
            # XXX: we override the find_key() because we want to be
            # interactive but that's ugly as hell - find_key() should
            # take a callback maybe?
            # 1.a) from the local keyring
            self.msui.log(_('looking for key %s in your keyring')
                          % self.msui.pattern)
            self.msui.keyring.context.set_option('export-options',
                                                 'export-minimal')
            if self.msui.tmpkeyring.import_data(self.msui.keyring.export_data(self.msui.pattern)):
                # for some reason, we were holding on to a lock
                # already, release it
                gtk.gdk.threads_leave()
                self.msui.copy_secrets()
                self.msui.sign_key()
                self.msui.export_key()
                self.resume_capture()
                # XXX: also ugly, reindent everything instead
                return

            # 1.b) if allowed (@todo), from the keyservers
            if self.msui.options.keyserver is not None:
                self.msui.tmpkeyring.context.set_option('keyserver', self.msui.options.keyserver)
            command = self.msui.tmpkeyring.context.build_command(['recv-keys', self.msui.pattern])
            self.msui.log('cmd: ' + str(command))
            self.dialog = gtk.Dialog(title=_('Please wait'), parent=None, flags=gtk.DIALOG_MODAL, buttons=None)
            self.dialog.add_button('gtk-cancel', gtk.RESPONSE_CANCEL)
            message = gtk.Label(_('Retrieving public key from server...'))
            message.show()
            self.progressbar = gtk.ProgressBar()
            self.progressbar.show()
            self.dialog.vbox.pack_start(message, True, True, 5)
            self.dialog.vbox.pack_start(self.progressbar, False, False, 5)
            self.dialog.set_size_request(250, 100)
            self.keep_pulsing = True
            proc = subprocess.Popen(command, 0, None,  # nosec
                                    subprocess.PIPE,
                                    subprocess.PIPE,
                                    subprocess.PIPE)
            gobject.child_watch_add(proc.pid, self.watch_out_callback)
            gobject.timeout_add(100, self.update_progress_callback)
            if self.dialog.run() == gtk.RESPONSE_CANCEL:
                proc.kill()
        else:
            self.msui.warn(_('data found in barcode does not match a OpenPGP fingerprint pattern: %s') % data)
            self.resume_capture()

    def resume_capture(self):
        """restart capture"""
        self.zbarframe.remove(self.capture)
        try:
            self.zbar.set_video_enabled(True)
        except AttributeError:  # nosec
            # the "zbar" is not a video frame capture,
            # webcam probably disable, ignore
            pass
        self.zbarframe.add(self.zbar)
        self.capture = None

    def destroy(self, widget, data=None):
        """close the application"""
        try:
            self.zbar.set_video_enabled(False)
        except AttributeError:  # nosec
            # the "zbar" is not a video frame capture,
            # webcam probably disable, ignore
            pass
        del self.msui
        gtk.main_quit()

    def about_dialog(self, widget, data=None):
        dialog = gtk.AboutDialog()
        dialog.set_version(monkeysign.__version__)
        dialog.set_copyright(monkeysign.__copyright__)
        dialog.set_comments(monkeysign.__description__)
        dialog.set_license(monkeysign.__license__)
        dialog.set_website(monkeysign.__website__)
        dialog.set_authors(monkeysign.__authors__)
        dialog.set_documenters(monkeysign.__documenters__)
        translators_str = "\n".join(monkeysign.__translators__)
        dialog.set_translator_credits(translators_str)
        dialog.show()
        dialog.run()
        dialog.destroy()

    def documentation_dialog(self, widget, data=None):
        webbrowser.open(monkeysign.__documentation__, autoraise=True)

    def qr_code_event(self, thing, event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            self.qr_code_right_click_menu(event)

    def qr_code_right_click_menu(self, event):
        menu = gtk.Menu()
        menu_items = [
            (_('_Save QR code as...'), self.save_qrcode),
            (_('_Print QR code...'), self.print_op),
        ]

        for item in menu_items:
            new_menu_item = gtk.MenuItem(item[0])
            new_menu_item.connect('activate', item[1])
            new_menu_item.show()
            menu.append(new_menu_item)

        menu.popup(None, None, None, event.button, event.time)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Test suite for the basic user interface class.
"""

from email.header import Header
import locale
import unittest
import os
import pkg_resources
from pkg_resources import ResolutionError
import sys
import re
from StringIO import StringIO
import tempfile
try:
    from unittest.mock import Mock
except ImportError:
    try:
        from mock import Mock
    except:
        Mock = False

def which2(cmd):
    """naive implementation of which for Python 2"""
    path = os.environ.get("PATH", os.defpath)
    if not path:
        return None
    path = path.split(os.pathsep)
    for dir in path:
        name = os.path.join(dir, cmd)
        if os.path.exists(name):
            return name
    return None

try:
    from shutil import which  # only in py3
except ImportError:
    which = which2

sys.path.insert(0, os.path.dirname(__file__) + '/..')

from monkeysign.ui import MonkeysignUi, EmailFactory, MonkeysignArgumentParser
from monkeysign.gpg import TempKeyring, GpgRuntimeError

import test_lib
# optimized because called often
from test_lib import find_test_file, skipIfDatePassed, skipUnlessNetwork


class CliBaseTest(unittest.TestCase):
    def setUp(self):
        self.argv = sys.argv
        sys.argv = [ 'monkeysign', '--no-mail' ]

    def execute(self):
        '''execute the monkeysign script directly

        there are two ways to do this without subprocess:

        1. execfile(somepath), which we use when we run from the
        source tree directly

        2. run_script, which we use when we are installed through setup.py
        '''
        directory, basename = os.path.split(self.argv[0])
        path, directory = os.path.split(os.path.realpath(directory))
        if directory == 'scripts' or basename == 'setup.py':
            # default to run from source dir. this is necessary for
            # ./scripts/monkeysign --test and ./setup.py test
            execfile(os.path.dirname(__file__) + '/../../scripts/monkeysign')
        else:
            try:
                pkg = pkg_resources.get_distribution('monkeysign')
                pkg.run_script('monkeysign', globals())
            except pkg_resources.ResolutionError:
                # package exists, but script not in distribution
                # this happens when wheel is installed by pip
                execfile(which('monkeysign'))

    def write_to_callback(self, stdin, callback):
        r, w = os.pipe()
        pid = os.fork()
        if pid:
            # parent
            os.close(w)
            os.dup2(r, 0) # make stdin read from the child
            oldstdout = sys.stdout
            sys.stdout = open('/dev/null', 'w') # silence output
            callback(self)
            sys.stdout = oldstdout
        else:
            # child
            os.close(r)
            w = os.fdopen(w, 'w')
            w.write(stdin) # say whatever is needed to msign-cli
            w.flush()
            os._exit(0)

    def tearDown(self):
        sys.argv = self.argv


@unittest.skipUnless(Mock, 'mock library missing')
class CliTestCase(CliBaseTest):
    def setUp(self):
        CliBaseTest.setUp(self)

        def sysinfo():
            return []

        ui = Mock()
        ui.sysinfo = MonkeysignUi.sysinfo
        ui.epilog = '%s'
        ui.usage = ''
        self.parser = MonkeysignArgumentParser(ui)

    def test_call_usage(self):
        stderrbak = sys.stderr
        with open('/dev/null', 'a') as sys.stderr:
            with self.assertRaises(SystemExit):
                self.execute()
        sys.stderr = stderrbak

    def test_call_version(self):
        """test to see if we output monkeysign and gpg versions"""
        sys.argv = ['monkeysign', '--version']
        stderrbak = sys.stderr
        sys.stderr = StringIO()
        with self.assertRaises(SystemExit):
            self.execute()
        self.assertIn('Monkeysign: ', sys.stderr.getvalue())
        self.assertIn('GnuPG: ', sys.stderr.getvalue())
        sys.stderr = stderrbak

    def test_configs(self):
        """test configuration file support"""
        config1 = tempfile.NamedTemporaryFile()
        config2 = tempfile.NamedTemporaryFile()
        config1.write('user=foo')
        config2.write('user=bar')
        config1.flush()
        config2.flush()
        self.parser.configs = [config1.name, config2.name]
        args = self.parser.parse_args()
        self.assertEqual(args.user, 'foo', 'first config wins')
        args = self.parser.parse_args(['--user', 'quux'])
        self.assertEqual(args.user, 'quux', 'commandline wins')

    def test_write_configs(self):
        """test how we can write config files"""
        args = self.parser.parse_args(['--local', '--smtpserver', 'localhost', '--tls', 'foo'])
        # first test parts of save_config()
        lines = self.parser.args_to_config(args)
        out = StringIO()
        self.parser.write_config(lines, out)
        out = out.getvalue()
        self.assertIn('smtpserver=localhost', out)
        self.assertIn('local', out)
        self.assertIn('tls', out)
        self.assertNotIn('foo', out)
        config = tempfile.NamedTemporaryFile()
        # then save_config() itself
        self.assertTrue(self.parser.save_config(args, config, close=False))
        config.flush()
        args = self.parser.parse_args(['@'+config.name])
        self.assertEqual(args.smtpserver, 'localhost')
        self.assertTrue(args.tls)
        self.assertTrue(args.local)


@skipIfDatePassed('2017-06-01T00:00:00UTC')
class CliTestDialog(CliBaseTest):
    def setUp(self):
        CliBaseTest.setUp(self)
        self.gpg = TempKeyring()
        os.environ['GNUPGHOME'] = self.gpg.homedir
        self.assertTrue(self.gpg.import_data(open(find_test_file('7B75921E.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))

        sys.argv += [ '-u', '96F47C6A', '--', '7B75921E' ]

    @test_lib.skipUnlessUnicodeLocale()
    def test_sign_fake_keyring(self):
        """test if we can sign a key on a fake keyring"""
        def callback(self):
            self.execute()
        self.write_to_callback("y\ny\n", callback) # just say yes

    @test_lib.skipUnlessUnicodeLocale()
    def test_sign_one_uid(self):
        """test if we can sign only one keyid"""
        def callback(self):
            self.execute()
        self.write_to_callback("n\n1\ny\n", callback) # just say yes

    @test_lib.skipUnlessUnicodeLocale()
    def test_two_empty_responses(self):
        """test what happens when we answer nothing twice

this tests for bug #716675"""
        def callback(self):
            with self.assertRaises(EOFError):
                self.execute()
        self.write_to_callback("\n\n", callback) # say 'default' twice


@skipIfDatePassed('2017-06-01T00:00:00UTC')
class CliTestSpacedFingerprint(CliTestDialog):
    def setUp(self):
        CliTestDialog.setUp(self)
        sys.argv.pop() # remove the uid from parent class
        sys.argv += '8DC9 01CE 6414 6C04 8AD5  0FBB 7921 5252 7B75 921E'.split()

class BaseTestCase(unittest.TestCase):
    pattern = None
    args = []

    def setUp(self):
        self.args = [ '--no-mail' ] + self.args + [ x for x in sys.argv[1:] if x.startswith('-') ]
        if self.pattern is not None:
            self.args += [ self.pattern ]
        # do not read system-wide configs in tests
        MonkeysignArgumentParser.configs = []
        self.ui = MonkeysignUi(self.args)
        self.ui.keyring = TempKeyring()
        self.ui.prepare() # needed because we changed the base keyring

class BasicTests(BaseTestCase):
    pattern = '7B75921E'

    def setUp(self):
        BaseTestCase.setUp(self)
        self.homedir = self.ui.tmpkeyring.homedir

    def test_cleanup(self):
        del self.ui
        self.assertFalse(os.path.exists(self.homedir))


@skipIfDatePassed('2017-06-01T00:00:00UTC')
class SigningTests(BaseTestCase):
    pattern = '7B75921E'

    def setUp(self):
        """setup a basic keyring capable of signing a local key"""
        BaseTestCase.setUp(self)
        self.assertTrue(self.ui.keyring.import_data(open(find_test_file('7B75921E.asc')).read()))
        self.assertTrue(self.ui.tmpkeyring.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.ui.keyring.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.ui.keyring.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))

    def test_find_key(self):
        """test if we can extract the key locally

this duplicates tests from the gpg code, but is necessary to test later functions"""
        self.ui.find_key()

    def test_copy_secrets(self):
        """test if we can copy secrets between the two keyrings

this duplicates tests from the gpg code, but is necessary to test later functions"""
        self.test_find_key()
        self.ui.copy_secrets()
        self.assertTrue(self.ui.keyring.get_keys(None, True, False))
        self.assertGreaterEqual(len(self.ui.keyring.get_keys(None, True, False)), 1)
        self.assertGreaterEqual(len(self.ui.keyring.get_keys(None, True, True)), 1)

    def test_sign_key(self):
        """test if we can sign the keys non-interactively"""
        self.test_copy_secrets()
        self.ui.sign_key()
        self.assertGreaterEqual(len(self.ui.signed_keys), 1)

    def test_sign_own_key(self):
        """Test that signing a user's own key fails"""
        self.ui.pattern = '96F47C6A'
        self.test_copy_secrets()
        self.ui.sign_key()
        self.assertEquals(len(self.ui.signed_keys), 0)
        self.ui.pattern = '7B75921E'

    def test_gpg_conf(self):
        """test if gpg.conf default-key works"""
        # this fails with GnuPG 2.1
        with open(os.path.join(self.ui.keyring.homedir, 'gpg.conf'), 'w') as f:
            f.write('default-key 3F94240C918E63590B04152E86E4E70A96F47C6A')
        # needed because we created a new gpg.conf
        self.ui.prepare()
        self.assertTrue(os.path.exists(os.path.join(self.ui.tmpkeyring.homedir, 'gpg.conf')))
        self.test_copy_secrets()
        self.ui.tmpkeyring.context.call_command(['list-secret-keys'])
        self.assertIn('3F94240C918E63590B04152E86E4E70A96F47C6A',
                      self.ui.tmpkeyring.context.stdout,
                      'tmpkeyring can see private keys')
        self.ui.sign_key()
        self.ui.tmpkeyring.context.call_command(['list-sigs', '7B75921E'])
        # this is the primary test key, it should have signed this
        self.assertIn('sig:::1:86E4E70A96F47C6A:',
                      self.ui.tmpkeyring.context.stdout)

    def test_multiple_secrets(self):
        """test if we pick the right key define in gpg.conf"""
        # configure gpg to use the second test key as a default key
        with open(os.path.join(self.ui.keyring.homedir, 'gpg.conf'), 'w') as f:
            f.write('default-key 323F39BD')
        self.ui.prepare()
        self.test_copy_secrets()
        self.ui.keyring.import_data(open(find_test_file('323F39BD.asc')).read())
        self.ui.keyring.import_data(open(find_test_file('323F39BD-secret.asc')).read())
        self.test_copy_secrets()
        self.ui.sign_key()
        self.ui.tmpkeyring.context.call_command(['list-sigs', '7B75921E'])
        # this is the primary test key, it shouldn't have signed this
        self.assertNotIn('sig:::1:86E4E70A96F47C6A:',
                         self.ui.tmpkeyring.context.stdout)
        # try to find a revoked uid that is signed
        for uid in self.ui.tmpkeyring.context.stdout.split('uid:'):
            if 'rev:' in uid:
                self.assertNotIn('sig:::1:A31E75E4323F39BD', uid)

    def test_create_mail_multiple(self):
        """test if exported keys contain the right uid"""
        self.test_sign_key()

        for fpr, key in self.ui.signed_keys.items():
            oldmsg = None
            for uid in key.uids.values():
                msg = EmailFactory(self.ui.tmpkeyring.export_data(fpr), fpr, uid.uid, 'unittests@localhost', 'devnull@localhost')
                if oldmsg is not None:
                    self.assertNotEqual(oldmsg.as_string(), msg.as_string())
                    self.assertNotEqual(oldmsg.create_mail_from_block().as_string(),
                                        msg.create_mail_from_block().as_string())
                    self.assertNotEqual(oldmsg.tmpkeyring.export_data(fpr),
                                        msg.tmpkeyring.export_data(fpr))
                oldmsg = msg
            self.assertIsNot(oldmsg, None)

    def test_export_key(self):
        """see if we export a properly encrypted key set"""
        messages = []
        # collect messages instead of warning the user
        self.ui.warn = messages.append
        self.test_sign_key()
        self.ui.export_key()
        self.assertIsNone(self.ui.export_key(), 'sends mail?')
        paste = messages.pop()
        self.assertNotIn('BEGIN PGP PUBLIC KEY BLOCK', paste,
                         'message not encrypted')
        self.assertIn('BEGIN PGP MESSAGE', paste, 'message not encrypted')
        self.assertNotIn('MIME-Version', paste,
                         'message to paste has weird MIME headers')

    def test_sendmail(self):
        """see if we can generate a proper commandline to send email"""
        self.test_sign_key()
        messages = []
        # collect messages instead of warning the user
        self.ui.warn = messages.append
        self.ui.options.nomail = False
        self.ui.options.user = 'unittests@localhost'
        self.ui.options.to = 'devnull@localhost'
        self.ui.options.mta = "dd status=none of='" + \
                              self.ui.keyring.homedir + "/%(to)s'"
        self.assertTrue(self.ui.export_key(), 'fails to send mail')
        filename = '%s/%s' % (self.ui.keyring.homedir, self.ui.options.to)
        self.assertGreater(os.path.getsize(filename), 0,
                           'mail properly created')
        self.assertIn('sent message to %s with dd' % self.ui.options.to,
                      messages.pop(),
                      'missing information to user')
        self.ui.options.to = 'devnull; touch bad'
        self.assertTrue(self.ui.export_key(),
                        'fails to send email to weird address')
        self.assertFalse(os.path.exists('bad'),
                         'vulnerable to command injection')

    def test_mua(self):
        self.test_sign_key()
        messages = []
        # collect messages instead of warning the user
        self.ui.warn = messages.append
        self.ui.options.nomail = False
        self.ui.options.user = 'unittests@localhost'
        self.ui.options.to = 'devnull@localhost'
        self.ui.options.mua = 'python ' + os.path.join(os.path.dirname(__file__),
                                           'mua-writer.py') \
            + " --to %(to)s --subject %(subject)s --body %(body)s" \
            + " --attach %(attach)s"
        self.assertTrue(self.ui.export_key(), 'fails to send mail')
        outputfile = self.ui.mailer_output.rstrip()
        self.assertTrue(os.path.exists(outputfile), 'file %s exists' % outputfile)
        with open(outputfile) as email:
            contents = email.read()
            self.assertIn('MIME', contents)
            self.assertIn(self.ui.options.to, contents)
            self.assertIn(self.ui.options.user, contents)
            try:
                subject = Header(EmailFactory.subject.encode('ascii')).encode()
            except UnicodeEncodeError:
                subject = Header(EmailFactory.subject, 'UTF-8').encode()
            self.assertIn(subject, contents)
            self.assertIn(EmailFactory.body, contents)
        os.unlink(outputfile)


@skipIfDatePassed('2017-06-01T00:00:00UTC')
class EmailFactoryTest(BaseTestCase):
    pattern = '7B75921E'

    def setUp(self):
        """setup a basic keyring capable of signing a local key"""
        BaseTestCase.setUp(self)
        self.assertTrue(self.ui.tmpkeyring.import_data(open(find_test_file('7B75921E.asc')).read()))
        self.assertTrue(self.ui.tmpkeyring.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.ui.tmpkeyring.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))

        self.email = EmailFactory(self.ui.tmpkeyring.export_data(self.pattern), self.pattern, 'Antoine Beaupré <anarcat@orangeseeds.org>', 'nobody@example.com', 'nobody@example.com')

    def test_cleanup_uids(self):
        """test if we can properly remove irrelevant UIDs"""
        for fpr, key in self.email.tmpkeyring.get_keys('7B75921E').iteritems():
            for u, uid in key.uids.iteritems():
                self.assertEqual(self.email.recipient, uid.uid)

    def test_mail_key(self):
        """test if we can generate a mail with a key inside"""
        message = self.email.create_mail_from_block()
        match = re.compile("""Content-Type: multipart/mixed; boundary="===============[0-9]*=="
MIME-Version: 1.0

--===============[0-9]*==
Content-Type: text/plain; charset="utf-8"
MIME-Version: 1.0
Content-Transfer-Encoding: quoted-printable

%s
--===============[0-9]*==
Content-Type: application/pgp-keys; name="signed-7B75921E.asc"
MIME-Version: 1.0
Content-Disposition: attachment; filename="signed-7B75921E.asc"
Content-Transfer-Encoding: 7bit
Content-Description: =\?utf-8\?q\?signed_OpenPGP_Key_7B75921E.*

-----BEGIN PGP PUBLIC KEY BLOCK-----
.*
-----END PGP PUBLIC KEY BLOCK-----

--===============[0-9]*==--""" % (self.email.body), re.DOTALL)
        self.assertRegexpMatches(message.as_string(), match)
        return message

    def test_wrap_crypted_mail(self):
        match = re.compile("""Content-Type: multipart/encrypted; protocol="application/pgp-encrypted";
 boundary="===============%s=="
MIME-Version: 1.0
Subject: .*
From: nobody@example.com
To: nobody@example.com

This is a multi-part message in OpenPGP/MIME format...
--===============%s==
Content-Type: application/pgp-encrypted; filename="signedkey.msg"
MIME-Version: 1.0
Content-Disposition: attachment; filename="signedkey.msg"

Version: 1
--===============%s==
Content-Type: application/octet-stream; filename="msg.asc"
MIME-Version: 1.0
Content-Disposition: inline; filename="msg.asc"
Content-Transfer-Encoding: 7bit

-----BEGIN PGP MESSAGE-----
.*
-----END PGP MESSAGE-----

--===============%s==--""" % tuple(['[0-9]*'] * 4), re.DOTALL)
        self.assertRegexpMatches(self.email.as_string(), match)

    def test_weird_from(self):
        """make sure we don't end up with spaces in our email address"""
        self.email = EmailFactory(self.ui.tmpkeyring.export_data(self.pattern), self.pattern, 'Antoine Beaupré <anarcat@orangeseeds.org>', 'Antoine Beaupré (home address) <anarcat@anarcat.ath.cx>', 'nobody@example.com')
        match = re.compile("""From: (([^ ]* )|("[^"]*" ))?<[^> ]*>$""", re.DOTALL | re.MULTILINE)
        self.assertRegexpMatches(self.email.as_string(), match)

class FakeKeyringTests(BaseTestCase):
    args = []
    pattern = '96F47C6A'

    def setUp(self):
        """we setup a fake keyring with the public key to sign and add our private keys"""
        BaseTestCase.setUp(self)
        self.ui.keyring.import_data(open(find_test_file('96F47C6A.asc')).read())

    def test_find_key(self):
        """test if we can find a key on the local keyring"""
        self.ui.find_key()


@skipUnlessNetwork()
class NonExistentKeyTests(BaseTestCase, test_lib.TestTimeLimit):
    """test behavior with a key that can't be found"""

    args = []
    # odds that a key with all zeros as fpr are low, unless something happens between PGP and bitcoins...
    pattern = '0000000000000000000000000000000000000000'

    def test_find_key(self):
        """find_key() should exit if the key can't be found on keyservers or local keyring"""
        try:
            with self.assertRaises(SystemExit):
                self.ui.find_key()
        except test_lib.AlarmException:
            raise unittest.case._ExpectedFailure(sys.exc_info())


@skipUnlessNetwork()
class KeyserverTests(BaseTestCase):
    args = ['--keyserver', 'pool.sks-keyservers.net']
    pattern = '7B75921E'

    def test_find_key(self):
        """this should find the key on the keyservers"""
        self.ui.find_key()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import errno
import locale
import sys
import os
import getpass

from monkeysign.ui import MonkeysignUi
import monkeysign.translation

class MonkeysignCli(MonkeysignUi):
    """sign a key in a safe fashion.

This command signs a key based on the fingerprint or user id
specified on the commandline, encrypt the result and mail it to the
user. This leave the choice of publishing the certification to that
person and makes sure that person owns the identity signed.

This program assumes you have gpg-agent configured to prompt for
passwords."""

    def main(self):
        """main code execution loop

        we expect to have the commandline parsed for us
        """

        MonkeysignUi.main(self)

        if not 'GPG_TTY' in os.environ:
            try:
                os.environ['GPG_TTY'] = os.ttyname(sys.stdin.fileno())
            except OSError as e:
                if e.errno == errno.ENOTTY:
                    self.warn(_('cannot find your TTY, GPG may freak out if you do not set the GPG_TTY environment'))
                else:
                    raise
            else:
                self.log(_('reset GPG_TTY to %s') % os.environ['GPG_TTY'])

        # 1. fetch the key into a temporary keyring
        self.find_key()

        # 2. copy the signing key secrets into the keyring
        self.copy_secrets()

        self.warn(_('Preparing to sign with this key\n\n%s') % self.signing_key)

        # 3. for every user id (or all, if -a is specified)
        # 3.1. sign the uid, using gpg-agent
        self.sign_key()

        # 3.2. export and encrypt the signature
        # 3.3. mail the key to the user
        # 3.4. optionnally (-l), create a local signature and import in
        #local keyring
        self.export_key()

        # 4. trash the temporary keyring
        # implicit

    def yes_no(self, prompt, default = None):
        ans = raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))
        while default is None and ans.lower() not in ["y", "n"]:
            ans = raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))
        if default: return default
        else: return ans.lower() == 'y'

    def acknowledge(self, prompt):
        prompt = _('press enter when %s') % prompt
        raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))

    def prompt_line(self, prompt):
        return raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))

    def prompt_pass(self, prompt):
        return getpass.getpass(prompt)

    def choose_uid(self, prompt, key):
        """present the user with a list of UIDs and let him choose one"""
        try:
            allowed_uids = []
            for uid in key.uidslist:
                allowed_uids.append(uid.uid)

            prompt += _(' (1-%d or full UID, control-c to abort): ') % len(allowed_uids)

            # workaround http://bugs.python.org/issue7768
            pattern = raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))
            while not (pattern in allowed_uids or (pattern.isdigit() and int(pattern)-1 in range(0,len(allowed_uids)))):
                print _('invalid uid')
                pattern = raw_input(prompt.encode(sys.stdout.encoding or locale.getpreferredencoding(True)))
            if pattern.isdigit():
                pattern = allowed_uids[int(pattern)-1]
            return pattern
        except KeyboardInterrupt:
            return False



# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import monkeysign
# gpg interface
from monkeysign.gpg import Keyring, TempKeyring, GpgRuntimeError, Context
import monkeysign.translation

import errno
# mail functions
from email.mime.multipart import MIMEMultipart
from email.mime.message import MIMEMessage
from email.mime.application import MIMEApplication
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.header import Header
from email.utils import parseaddr, formataddr
from email import Charset
import smtplib
import subprocess  # nosec

# system libraries
import argparse
import errno
import getpass
import os
import platform
import sys
import re
import shlex
try:
    from shlex import quote as cmd_quote
except ImportError:
    from pipes import quote as cmd_quote
import shutil
import socket
import socks
import tempfile


class RunTests(argparse._VersionAction):
    '''argparse Action handler to run tests

    we want to run them immediately because we don't want to force
    users to provide a pattern.

    '''

    def __call__(self, parser, namespace, values, option_string=None):
        '''run the test suite and exit with the number of failures'''
        import unittest
        sys.argv.remove('--test')
        if '--version' in sys.argv:
            sys.argv.remove('--version')
        testdir = os.path.join(os.path.dirname(__file__), 'tests')
        suite = unittest.TestLoader().discover(testdir)
        results = unittest.TextTestRunner().run(suite)
        parser.exit(len(results.errors))


class MonkeysignArgumentParser(argparse.ArgumentParser):
    # config files, in priority order (first is higher)
    configs = ['~/.config/monkeysign.conf', '/etc/monkeysign.conf']

    # standard header that needs to be present in the config file for
    # us to rewrite it
    header = """# this file was created by monkeysign
# comments will be lost on next write"""

    # those parameters will not be saved to the config file
    ephemeral = ['save', 'version', 'help', 'pattern', 'to', 'test']

    def __init__(self,
                 prog=None, usage=None, description=None, epilog=None,
                 parents=[], formatter_class=argparse.HelpFormatter,
                 prefix_chars='-', fromfile_prefix_chars=None,
                 argument_default=None, conflict_handler='error',
                 add_help=True, allow_abbrev=True,
                 ui=None):
        """create a standard commandline parser based on the given class
        documentation"""

        if ui is None:
            # mock object:
            ui = MonkeysignUi(args=[])

        argparse.ArgumentParser.__init__(self, description=ui.__doc__,
                                         usage=ui.usage,
                                         epilog=ui.epilog
                                         % " or ".join(self.configs),
                                         fromfile_prefix_chars='@',
                                         formatter_class=argparse.RawDescriptionHelpFormatter)
        self.add_argument('--version', action='version',
                            version="\n".join(ui.sysinfo()))
        self.add_argument('-d', '--debug', action='store_true',
                            help=_('request debugging information from GPG '
                                   'engine (lots of garbage)'))
        self.add_argument('-v', '--verbose', action='store_true',
                            help=_('explain what we do along the way'))
        self.add_argument('-n', '--dry-run', dest='dryrun', action='store_true',
                            help=_('do not actually do anything'))
        self.add_argument('--test', action=RunTests,
                          help=_('run test suite, exit with the number of failing tests'))
        self.add_argument('-u', '--user', nargs='+',
                            help=_('user id to sign the key with (equivalent '
                                   'to GPG\'s --local-user option)'))
        self.add_argument('--certlevel', '--cert-level',
                            help=_('certification level to sign the key with '
                                   '(equivalent to GPG\'s '
                                   '--default-cert-level)'))
        self.add_argument('-l', '--local', action='store_true',
                            help=_('import in normal keyring a local '
                                   'certification'))
        self.add_argument('-k', '--keyserver',
                            help=_('keyserver to fetch keys from'))
        self.add_argument('--tor', action='store_true',
                          help=_('use Tor to get keys and send email'))
        self.add_argument('--socks', metavar='HOST[:PORT]',
                          help=_('connect to SOCKS proxy'))
        self.add_argument('-s', '--smtpserver', '--smtp',
                            help=_('SMTP server to use, use a colon to specify '
                                   'the port number if non-default (%(port)d).'
                                   ' willl attempt to use STARTTLS to secure '
                                   'the connexion and fail if unsupported '
                                   '(default: deliver using the --mta '
                                   'command)') %
                            {'port': smtplib.SMTP_PORT})
        self.add_argument('--tls', action='store_true',
                            help=_('use a complete TLS connexion instead of '
                                   'using STARTTLS to upgrade the connexion. '
                                   'will change the default SMTP port to '
                                   '%(port)d')
                            % {'port': smtplib.SMTP_SSL_PORT})
        self.add_argument('--smtpuser',
                            help=_('username for the SMTP server '
                                   '(default: no user)'))
        self.add_argument('--smtppass',
                            help=_('password for the SMTP server '
                                   '(default: prompted, if --smtpuser is '
                                   'specified)'))
        self.add_argument('--mta', default="/usr/sbin/sendmail -t",
                            help=_('command to use to send mail, recipient is '
                                   'passed on the commandline in the "%%(to)s"'
                                   ' field, or the command must parse the '
                                   '"To:" header (default: %(default)s)'))
        default_mua = "xdg-email --utf8 --to '%(to)s' --subject '%(subject)s' --body '%(body)s' --attach '%(attach)s'"
        self.add_argument('--mua', nargs='?', default=None, const=default_mua,
                          help=_('Mail User Agent to use to send mail. all '
                                 'parameters are passed on the commandline,'
                                 'overrides --mta. '
                                 '(default: %(const)s when specified)'))
        self.add_argument('--nomail', '--no-mail', action='store_true',
                            help=_('do not send email at all'))
        self.add_argument('-t', '--to',
                            help=_('override destination email for testing '
                                   '(default: send individually encrypted '
                                   'email to each uid chosen)'))
        self.add_argument('--save', action='store_true',
                          help=_('save the given commandline parameters to the config file'))
        if 'cli' in ui.__class__.__name__.lower():
            nargs = '+'
        else:
            nargs = '*'
        self.add_argument('pattern', nargs=nargs,
                            help=_('pattern of the key to sign (fingerprint,'
                                   ' user id, etc)'))

    def convert_arg_line_to_args(self, arg_line):
        """parse a config file"""
        # skip whitespace and commented lines
        if re.match('^(#|[\s]*$)', arg_line):
            return []
        else:
            # all lines are assumed to be options
            # this is important otherwise an attacker could inject key
            # material in the config file and silently have his key be
            # signed
            return ['--' + arg_line]

    def parse_args(self, args=None, namespace=None):
        """override builtin argument parsing to also parse config files"""
        configs = map(lambda x: os.path.expanduser(x), self.configs)
        for conf in configs:
            try:
                with open(conf, 'r'):
                    if args:
                        args.insert(0, '@' + conf)
                    else:
                        sys.argv.insert(1, '@' + conf)
            except IOError:
                pass
        args = argparse.ArgumentParser.parse_args(self,
                                                  args=args,
                                                  namespace=namespace)
        # XXX: a bit clunky because the cli expects this to be the
        # output of parse_args() while the GTK ui expects this to be
        # populated as a string, later
        if len(args.pattern) < 1:
            args.pattern = None
        else:
            # accept space-separated fingerprints
            args.pattern = "".join(args.pattern)
        # accept space-separated fingerprints for user as well
        if args.user is not None:
            args.user = "".join(args.user)
        if args.save:
            self.save_config(args)
        if args.local:
            args.nomail = True
        if args.tor and args.socks:
            self.error('specify only one of --tor or --socks')
        if args.tor:
            args.socks = 'localhost:9050'
        if args.socks:
            args.socks = args.socks.split(':')
            if len(args.socks) < 2:
                # hardcode default port number
                args.socks += [socks.DEFAULT_PORTS[socks.SOCKS5]]
            args.socks[1] = int(args.socks[1])
            args.socks = tuple(args.socks)
        return args

    def save_config(self, args, config=None, close=True):
        """save given namespace to default configuration file

        config is a filehandle that will be written to. if None, the
        first element in the configs array will be opened and written
        to.

        the config file will not be written to if it doesn't contain
        the magic header

        performs no exception handling. returns True if all is well

        the config file will be closed unless close=False (used for
        unit tests)
        """
        lines = self.args_to_config(args)
        if config is None:
            config = open(os.path.expanduser(self.configs[0]),
                          'w+')
        contents = config.read()
        if len(contents) and self.header not in contents:
            if close:
                config.close()
            return False
        config.truncate(0)
        self.write_config(lines, config)
        if close:
            config.close()
        # self.ui.log() except that we don't have logging yet
        return True

    def args_to_config(self, args):
        """return given namespace as a configuration file"""
        lines = []
        for action in self._actions:
            if action.dest in self.ephemeral:
                continue
            val = vars(args)[action.dest]
            if val != action.default:
                if isinstance(action, argparse._StoreAction):
                    lines.append('%s=%s' % (action.dest, val))
                else:
                    lines.append(action.dest)
        return lines

    def write_config(self, config, fileobj):
        """take the list of lines and write it to the config file

        we implement our own here because:

        1. ConfigParser is not API stable across Py2/3
        2. ConfigParser is unnecessarily hard to use and complex
        3. ConfigParser requires sections which we don't need
        4. alternatives like ConfigObj are not in the standard library
        5. this whole config file writer is about 30 lines long
        """
        return fileobj.write(self.header + "\n" + "\n".join(config) + "\n")


class MonkeysignUi(object):
    """User interface abstraction for monkeysign.

    This aims to factor out a common pattern to sign keys that is used
    regardless of the UI used.

    This is mostly geared at console/text-based and X11 interfaces,
    but could also be ported to other interfaces (touch-screen/phone
    interfaces would be interesting).

    The actual process is in main(), which outlines what the
    subclasses of this should be doing.

    You should have a docstring in derived classes, as it will be
    added to the 'usage' output.
    """

    # what gets presented to the user in the usage (first and last lines)
    # default is to use the OptionParser's defaults
    # the 'docstring' above is the long description
    # XXX: deprecated, argparse is flexible enough that we do not need
    # this anymore
    usage = None
    epilog = """an arbitrary configuration file may also be supplied on the
commandline with a @ prefix (e.g. @foo.conf). the following
configuration files are parsed by default: %s"""

    @classmethod
    def sysinfo(cls):
        info = []
        info.append("Monkeysign: %s" % monkeysign.__version__)
        info.append('Load path: %s'
                    % os.path.realpath(os.path.dirname(__file__)))
        info.append("%s: %s (%s %s)"
                    % (platform.python_implementation(),
                       platform.python_version(),
                       platform.python_compiler(),
                       " ".join(platform.python_build())))
        info.append("Kernel: %s" % " ".join(platform.uname()))
        if 'linux' in platform.system().lower():
            distribution = " ".join(platform.linux_distribution()).rstrip()
        else:
            distribution = ''
        info.append("Operating system: %s (%s)"
                    % (distribution, platform.system()))
        info.append('PID: %d, CWD: %s' % (os.getpid(), os.getcwd()))
        info.append('Command: %r' % sys.argv)
        try:
            info.append('GnuPG: %s' % Context().version())
        except OSError as e:
            if e.errno == errno.ENOENT:
                info.append('GnuPG: error! %s: %s'
                            % (Context.gpg_binary, str(e)))
        return info

    def __init__(self, args = None):
        # the options that determine how we operate, from the parse_args()
        self.options = {}

        # the key we are signing, can be a keyid or a uid pattern
        self.pattern = None

        # the regular keyring we suck secrets and maybe the key to be signed from
        self.keyring = Keyring()

        # the temporary keyring we operate in, actually initialized in prepare()
        # this is because we want the constructor to just initialise
        # data structures and not write any data
        self.tmpkeyring = None

        # the fingerprints that we actually signed
        self.signed_keys = {}

        # temporary, to keep track of the OpenPGPkey we are signing
        self.signing_key = None

        self.parser = MonkeysignArgumentParser(ui=self)
        self.options = self.parser.parse_args(args=args)
        self.pattern = self.options.pattern

        # set a default logging mechanism
        self.logfile = sys.stderr
        self.log(_('Initializing UI'))

        # create the temporary keyring
        # XXX: i would prefer this to be done outside the constructor
        self.prepare()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        # this is implicit in the garbage collection, but tell the user anyways
        self.log(_('deleting the temporary keyring %s') % self.tmpkeyring.homedir)

        if exc_type is NotImplementedError:
            self.abort(str(exc_value))

    def prepare(self):
        # initialize the temporary keyring directory
        self.tmpkeyring = TempKeyring()

        if self.options.debug:
            self.tmpkeyring.context.debug = self.logfile
            self.keyring.context.debug = self.logfile
        if self.options.keyserver is not None:
            self.tmpkeyring.context.set_option('keyserver', self.options.keyserver)
        if self.options.user is not None:
            self.tmpkeyring.context.set_option('local-user', self.options.user)
        if self.options.certlevel is not None:
            self.tmpkeyring.context.set_option('default-cert-level', self.options.certlevel)
        self.tmpkeyring.context.set_option('secret-keyring', self.keyring.homedir + '/secring.gpg')

        if self.options.socks:
            # test if tor is running, as a safety precaution
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect(self.options.socks)
                s.close()
            except socket.error as e:
                m = list(self.options.socks)
                if self.options.tor:
                    m += [str(e) + _(' - is Tor running?')]
                else:
                    m += [str(e)]
                self.abort(_('cannot open SOCKS proxy %s:%d: %s') % m)
            else:
                self.log(_('SOCKS proxy %s:%d appears operational') % self.options.socks)
            # this should be --use-tor, but:
            # 1. that is available only in undetermined versions of GPG 2.1.x
            # 2. it is not available in GPG 1.x legacy, which we still support
            # 3. it is actually a flag to dirmngr, not gpg, so it
            # requires writing to a new config file
            ks_opts = 'no-auto-key-retrieve,no-try-dns-srv,http-proxy=socks5-hostname://%s:%d' % self.options.socks
            self.tmpkeyring.context.set_option('keyserver-options', ks_opts)
        # copy the gpg.conf from the real keyring
        try:
            self.log(_('copying your gpg.conf in temporary keyring'))
            shutil.copy(self.keyring.homedir + '/gpg.conf', self.tmpkeyring.homedir)
        except IOError as e:
            # no such file or directory is alright: it means the use
            # has no gpg.conf (because we are certain the temp homedir
            # exists at this point)
            if e.errno != errno.ENOENT:
                raise

        # install the gpg agent socket for GnuPG 2.1 because
        # --secret-keyring silently fails
        # this is apparently how we should do things:
        # https://lists.gnupg.org/pipermail/gnupg-devel/2015-January/029301.html
        # cargo-culted from caff, thanks guilhem!
        src = self.keyring.get_agent_socket()
        dst = self.tmpkeyring.get_agent_socket()
        self.log(_('installing symlinks for sockets from %s to %s') % (src, dst))
        try:
            os.unlink(dst)
        except OSError as e:
            if e.errno == errno.ENOENT:
                pass
            else:
                raise
        os.symlink(src, dst)

    def main(self):
        """
        General process
        ===============

        1. fetch the key into a temporary keyring
        1.a) if allowed (@todo), from the keyservers
        1.b) from the local keyring (@todo try that first?)
        2. copy the signing key secrets into the keyring
        3. for every user id (or all, if -a is specified)
        3.1. sign the uid, using gpg-agent
        3.2. export and encrypt the signature
        3.3. mail the key to the user
        3.4. optionnally (-l), create a local signature and import in
        local keyring
        4. trash the temporary keyring
        """
        pass # we don't do anything because we allow for interactive process

    def abort(self, message):
        """show a message to the user and abort program"""
        sys.exit(message)

    def warn(self, message):
        """display an warning message

this should not interrupt the flow of the program, but must be visible to the user"""
        print message.encode('utf-8')

    def log(self, message):
        """log an informational message if verbose"""
        if self.options.verbose or self.options.debug: print >>self.logfile, message

    def yes_no(self, prompt, default = True):
        """default UI is not interactive, so we assume yes all the time"""
        return True

    def acknowledge(self, prompt):
        return True

    def choose_uid(self, prompt, uids):
        raise NotImplementedError('choosing not implemented in base class')

    def prompt_line(self, prompt):
        raise NotImplementedError('prompting for a line not implemented in base class')

    def prompt_pass(self, prompt):
        raise NotImplementedError('prompting for a password not implemented in base class')

    def find_key(self):
        """find the key to be signed somewhere"""
        # 1.b) from the local keyring
        self.log(_('looking for key %s in your keyring') % self.pattern)
        if not self.tmpkeyring.import_data(self.keyring.export_data(self.pattern)):
            self.log(_('key not in local keyring'))

            # 1.a) if allowed, from the keyservers
            self.log(_('fetching key %s from keyserver') % self.pattern)

            if not re.search('^[0-9A-F]*$', self.pattern, re.IGNORECASE): # this is not a keyid
                # the problem here is that we need to implement --search-keys, and it's a pain
                raise NotImplementedError(_('please provide a keyid or fingerprint, uids are not supported yet'))

            if not self.tmpkeyring.fetch_keys(self.pattern):
                self.abort(_('could not find key %s in your keyring or keyservers') % self.pattern)

    def copy_secrets(self):
        """import secret keys (but only the public part) from your keyring

we use --secret-keyring instead of copying the secret key material,
but we still need the public part in the temporary keyring for this to
work.

we copy all keys because we do not want to guess which keys gpg will
chose. it could vary based on default-key, for example, or some weird
ordering.
        """
        self.log(_('copying your public key to temporary keyring in %s') % self.tmpkeyring.homedir)
        # detect the proper uid
        keys = self.keyring.get_keys(self.options.user, True, False)

        for fpr, key in keys.iteritems():
            self.log(_('found secret key: %s') % key)
            if not key.invalid and not key.disabled and not key.expired and not key.revoked:
                self.signing_key = key
                # export public key material associated with all private keys
                # XXX: we should only do export-minimal here, but passing options down is a pain.
                if not self.tmpkeyring.import_data(self.keyring.export_data(key.fpr)):
                    self.warning(_('could not export public key material of private key %s') % key.fpr)

        if self.signing_key is None:
            self.abort(_('could not find public key material for any private key, do you have an OpenPGP key?'))

    def sign_key(self):
        """sign the key uids, as specified"""

        keys = self.tmpkeyring.get_keys(self.pattern)

        # this shouldn't happen unless caller forgot to call copy_secrets
        assert(keys is not None)  # nosec
        self.log(_('found %d keys matching your request') % len(keys))

        secret_keys = self.keyring.get_keys(self.options.user, True, False)

        for key in keys:
            # Make sure the user isn't signing their own key
            for secret_key in secret_keys.values():
                if keys[key] == secret_key:
                    self.warn(_('That is your own key, so it is already certified'))
                    return False

            alluids = self.yes_no(_("""\
Signing the following key

%s

Sign all identities? [y/N] \
""") % keys[key], False)

            self.chosen_uid = None
            if alluids:
                pattern = keys[key].fpr
            else:
                pattern = self.choose_uid(_('Choose the identity to sign'), keys[key])
                if not pattern:
                    self.log(_('no identity chosen'))
                    return False
                if not self.options.to:
                    self.options.to = pattern
                self.chosen_uid = pattern

            if not self.options.dryrun:
                if not self.yes_no(_('Really sign key? [y/N] '), False):
                    continue
                if not self.tmpkeyring.sign_key(pattern, alluids):
                    self.warn(_('key signing failed'))
                else:
                    self.signed_keys[key] = keys[key]
                if self.options.local:
                    self.log(_('making a non-exportable signature'))
                    self.tmpkeyring.context.set_option('export-options', 'export-minimal')

                    # this is inefficient - we could save a copy if we would fetch the key directly
                    if not self.keyring.import_data(self.tmpkeyring.export_data(self.pattern)):
                        self.abort(_('could not import public key back into public keyring, something is wrong'))
                    if not self.keyring.sign_key(pattern, alluids, True):
                        self.warn(_('local key signing failed'))

    def export_key(self):
        if self.options.user is not None and '@' in self.options.user:
            from_user = self.options.user
        else:
            from_user = self.signing_key.uidslist[0].uid

        if len(self.signed_keys) < 1:
            self.warn(_('no key signed, nothing to export'))
        
        for fpr, key in self.signed_keys.items():
            if self.chosen_uid is None:
                for uid in key.uids.values():
                    try:
                        msg = EmailFactory(self.tmpkeyring.export_data(fpr), fpr, uid.uid, from_user, self.options.to)
                    except GpgRuntimeError as e:
                        self.warn(_('failed to create email: %s') % e)
                        break
                    return self.sendmail(msg)
            else:
                try:
                    msg = EmailFactory(self.tmpkeyring.export_data(fpr), fpr, self.chosen_uid, from_user, self.options.to)
                except GpgRuntimeError as e:
                    self.warn(_('failed to create email: %s') % e)
                    break
                return self.sendmail(msg)

    def sendmail(self, msg):
            """actually send the email

expects an EmailFactory email, but will not mail if nomail is set"""
            if self.options.smtpserver is not None and not self.options.nomail:
                if self.options.dryrun: return True
                # no test, because we'd need to ship a socks server
                # we assume pysocks is correctly tested, basically:
                # https://github.com/Anorov/PySocks
                if self.options.socks:
                    socks.setdefaultproxy(socks.SOCKS5, *self.options.socks)
                    socks.wrapmodule(smtplib)
                if self.options.tls:
                    server = smtplib.SMTP_SSL()
                else:
                    server = smtplib.SMTP()
                server.set_debuglevel(self.options.debug)
                # to be nicer to users, we could catch socket.error exceptions from
                # server.connect() here and display a meaningful message to stderr.
                try:
                    (code, srvmsg) = server.connect(self.options.smtpserver)
                except (socket.error, socket.timeout, socks.SOCKS5Error) as e:
                    self.abort(_('Error connecting to SMTP server %s: %s') % (self.options.smtpserver, e))
                if code != 220:
                    self.abort(_('Unexpected SMTP server error while talking to %s, code: %s (%s)') % (self.options.smtpserver, code, srvmsg))
                if not self.options.tls:
                    try:
                        server.starttls()
                    except smtplib.SMTPException:
                        self.warn(_('SMTP server does not support STARTTLS'))
                        if self.options.smtpuser:
                            self.abort(_('aborting authentication as credentials would have been sent in clear text'))
                if self.options.smtpuser:
                    if not self.options.smtppass:
                        self.options.smtppass = self.prompt_pass(_('enter SMTP password for server %s: ') % self.options.smtpserver)
                    server.login(self.options.smtpuser, self.options.smtppass)
                try:
                    server.sendmail(msg.mailfrom.encode('utf-8'), msg.mailto.encode('utf-8'), msg.as_string().encode('utf-8'))
                except smtplib.SMTPException as e:
                    self.warn(_('failed to send email: %s') % e)
                else:
                    self.warn(_('sent message through SMTP server %s to %s') % (self.options.smtpserver, msg.mailto))
                server.quit()
                return True
            elif not self.options.nomail:
                if self.options.dryrun: return True
                if self.options.mua:
                    keyfile = tempfile.NamedTemporaryFile()
                    # XXX: this should be part of the EmailFactory
                    keymaterial = msg.tmpkeyring.export_data(msg.keyfpr)
                    encrypted = msg.tmpkeyring.encrypt_data(keymaterial,
                                                            msg.keyfpr)
                    keyfile.write(encrypted)
                    keyfile.flush()
                try:
                    if self.options.mua:
                        command = [x % {'to': msg.mailto,
                                        'body': msg.body,
                                        'subject': msg.subject,
                                        'attach': keyfile.name}
                                   for x in shlex.split(self.options.mua)]
                    else:
                        command = [x % {'to': msg.mailto}
                                   for x in shlex.split(self.options.mta)]
                except ValueError as e:
                    if 'incomplete format' in str(e):
                        self.warn(_('invalid command (%s) specified to send email: %s')
                                  % (self.options.mua or self.options.mta, str(e)))
                        return False
                    else:
                        raise
                self.log('running command %s' % command)
                try:
                    p = subprocess.Popen(command,
                                         stdin=subprocess.PIPE,
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE)  # nosec
                except OSError as e:
                    self.warn(_('cannot find MTA %s, try specifying --mua, --mta or --smtp: %s')
                              % (self.options.mua, repr(e)))
                    return False

                if self.options.mua:
                    tosend = None
                else:
                    tosend = msg.as_string().encode('utf-8')
                self.mailer_output, self.mailer_error = p.communicate(tosend)
                if self.options.mua:
                    self.acknowledge('when you have finished writing the email')
                    keyfile.close()  # remove the tmpfile
                if p.returncode == 0:
                    self.warn(_('sent message to %(destination)s with %(command)s')
                              % {'destination': msg.mailto,
                                 'command': self.options.mua or self.options.mta})
                else:
                    self.warn(_('failed sending message to %(destination)s with %(command)s: %(error)s')
                              % {'destination': msg.mailto,
                                 'command': self.options.mua or self.options.mta,
                                 'error': self.mailer_output + self.mailer_error})
                return p.returncode == 0
            else:
                # okay, no mail, just dump the exported key then
                self.warn(_("""\
not sending email to %s, as requested, here's the encrypted signed public key you can paste in your email client:

%s""") % (msg.mailto, msg.encrypted()))


class EmailFactory:
    """email generator

this is a factory, ie. a class generating an object that represents
the email and when turned into a string, is the actual
mail.
"""

    # the email subject
    subject = _("Your signed OpenPGP key")

    # the email body
    body = _("""
Please find attached your signed OpenPGP key. You can import the
signed key by running each through `gpg --import`.

If you have multiple user ids, each signature was sent in a separate
email to each user id.

Note that your key was not uploaded to any keyservers. If you want
this new signature to be available to others, please upload it
yourself.  With GnuPG this can be done using:

    gpg --keyserver pool.sks-keyservers.net --send-key <keyid>

Regards,
""")

    def __init__(self, keydata, keyfpr, recipient, mailfrom, mailto):
        """email constructor

we expect to find the following arguments:

keydata: the signed public key material
keyfpr: the fingerprint of that public key
recipient: the recipient to encrypt the mail to
mailfrom: who the mail originates from
mailto: who to send the mail to (usually similar to recipient, but can be used to specify specific keyids"""
        (self.keyfpr, self.recipient, self.mailfrom, self.mailto) = (keyfpr, recipient, mailfrom.decode('utf-8'), mailto or recipient)
        self.mailto = self.mailto.decode('utf-8')
        # operate over our own keyring, this allows us to remove UIDs freely
        self.tmpkeyring = TempKeyring()
        # copy data over from the UI keyring
        self.tmpkeyring.import_data(keydata)
        # prepare for email transport
        self.tmpkeyring.context.set_option('armor')
        # this is necessary because we reimport keys from outside our
        # keyring, so gpg doesn't trust them anymore
        # but we know we do, so we ignore the trustdb
        self.tmpkeyring.context.set_option('trust-model', 'always')
        # remove UIDs we don't want to send
        self.cleanup_uids()
        # cleanup email addresses
        self.cleanup_emails()

    def cleanup_emails(self):
        # wrap real name in quotes
        self.mailfrom = re.sub(r'^(.*) <', r'"\1" <',
                               # trim comment from uid
                               re.sub(r' \([^)]*\)', r'',
                                      self.mailfrom))
        # same with mailto
        self.mailto = re.sub(r'^(.*) <', r'"\1" <',
                             re.sub(r' \([^)]*\)', r'',
                                    self.mailto))

    def cleanup_uids(self):
        """this will remove any UID not matching the 'recipient' set in the class"""
        for fpr, key in self.tmpkeyring.get_keys().iteritems():
            todelete = []
            for uid in key.uids.values():
                if self.recipient != uid.uid:
                    todelete.append(uid.uid)
            for uid in todelete:
                self.tmpkeyring.del_uid(fpr, uid)

    def encrypted(self):
        '''encrypted blob of the MIME multipart message

        this is designed to be attached as the main body of an email,
        but can also be copy-pasted anywhere, although the decrypted
        version will look a little ackward because it will include all
        sorts of MIME stuff

        but if proper headers are added to the message, it should be
        parsed okay by OpenPGP and MIME-capable clients.'''

        msg = self.create_mail_from_block().as_string()
        return self.tmpkeyring.encrypt_data(msg, self.keyfpr)

    def get_message(self):
        # first layer, seen from within:
        # an encrypted MIME message, made of two parts: the
        # introduction and the signed key material
        encrypted = self.encrypted()

        # the second layer up, made of two parts: a version number
        # and the first layer, encrypted
        return self.wrap_crypted_mail(encrypted)

    def __str__(self):
        return self.get_message().as_string().decode('utf-8')

    def as_string(self):
        return self.__str__()

    def create_mail_from_block(self):
        """
        a multipart/mixed message containing a plain-text message
        explaining what this is, and a second part containing PGP data
        """

        # Override python's weird assumption that utf-8 text should be encoded with
        # base64, and instead use quoted-printable (for both subject and body).  I
        # can't figure out a way to specify QP (quoted-printable) instead of base64 in
        # a way that doesn't modify global state. :-(
        # (taken from http://radix.twistedmatrix.com/2010/07/how-to-send-good-unicode-email-with.html)
        Charset.add_charset('utf-8', Charset.QP, Charset.QP, 'utf-8')

        text = MIMEText(self.body, 'plain', 'utf-8')
        filename = 'signed-%s.asc' % self.keyfpr # should be 0xkeyid.uididx.signed-by-0xkeyid.asc
        keypart = MIMEBase('application', 'pgp-keys', name=filename)
        keypart.add_header('Content-Disposition', 'attachment', filename=filename)
        keypart.add_header('Content-Transfer-Encoding', '7bit')
        keypart.add_header('Content-Description', (_('signed OpenPGP Key %s, uid %s') % (self.keyfpr, self.recipient.decode('utf-8'))))
        keypart.set_payload(self.tmpkeyring.export_data(self.keyfpr))
        return MIMEMultipart('mixed', None, [text, keypart])

    def wrap_crypted_mail(self, encrypted):
        p1 = MIMEBase('application', 'pgp-encrypted', filename='signedkey.msg')
        p1.add_header('Content-Disposition','attachment', filename='signedkey.msg')
        p1.set_payload('Version: 1')
        p2 = MIMEBase('application', 'octet-stream', filename='msg.asc')
        p2.add_header('Content-Disposition', 'inline', filename='msg.asc')
        p2.add_header('Content-Transfer-Encoding', '7bit')
        p2.set_payload(encrypted)
        msg = MIMEMultipart('encrypted', None, [p1, p2], protocol="application/pgp-encrypted")
        msg.preamble = _('This is a multi-part message in OpenPGP/MIME format...')
        try:
            msg['Subject'] = Header(self.subject.encode('ascii'))
        except UnicodeEncodeError:
            msg['Subject'] = Header(self.subject, 'UTF-8')
        name, address = parseaddr(self.mailfrom)
        msg['From'] = formataddr((Header(name.encode('utf-8'), 'UTF-8').encode(), address))
        name, address = parseaddr(self.mailto)
        msg['To'] = formataddr((Header(name.encode('utf-8'), 'UTF-8').encode(), address))
        return msg



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""General test suite for the GPG API.

Tests that require network access should go in test_network.py.
"""

import sys, os, shutil
from StringIO import StringIO
import unittest
import tempfile
import re

sys.path.insert(0, os.path.dirname(__file__) + '/..')

from monkeysign.gpg import *
from test_lib import find_test_file, skipUnlessUnicodeLocale, skipIfDatePassed

class TestContext(unittest.TestCase):
    """Tests for the Context class.

    Those should be limited to talking to the GPG binary, not
    operating on actual keyrings or GPG data."""

    # those need to match the options in the Gpg class
    options = Context.options

    # ... and this is the rendered version of the above
    rendered_options = ['gpg', '--command-fd', '0', '--with-fingerprint', '--list-options', 'show-sig-subpackets,show-uid-validity,show-unusable-uids,show-unusable-subkeys,show-keyring,show-sig-expire', '--batch', '--fixed-list-mode', '--no-tty', '--with-colons', '--use-agent', '--status-fd', '2', '--quiet' ]

    def setUp(self):
        self.gpg = Context()

    def test_plain(self):
        """make sure other instances do not poison us"""
        d = Context()
        d.set_option('homedir', '/var/nonexistent')
        self.assertNotIn('homedir', self.gpg.options)

    def test_set_option(self):
        """make sure setting options works"""
        self.gpg.set_option('armor')
        self.assertIn('armor', self.gpg.options)
        self.gpg.set_option('keyserver', 'foo.example.com')
        self.assertDictContainsSubset({'keyserver': 'foo.example.com'}, self.gpg.options)

    def test_command(self):
        """test various command creation

        if this fails, it's probably because you added default options
        to the tested class without adding them in the test class
        """
        c = self.rendered_options + ['--version']
        c2 = self.gpg.build_command(['version'])
        self.assertItemsEqual(c, c2)
        c = self.rendered_options + ['--export', 'foo']
        c2 = self.gpg.build_command(['export', 'foo'])
        self.assertItemsEqual(c, c2)

    def test_version(self):
        """make sure version() returns something"""
        self.assertTrue(self.gpg.version())

    def test_seek_debug(self):
        """test if seek actually respects debug"""
        self.gpg.debug = True # should yield an attribute error, that's fine
        with self.assertRaises(AttributeError):
            self.gpg.seek(StringIO('test'), 'test')
        # now within a keyring?
        k = TempKeyring()
        k.context.debug = True
        with self.assertRaises(AttributeError):
            k.import_data(open(find_test_file('96F47C6A.asc')).read())

class TestTempKeyring(unittest.TestCase):
    """Test the TempKeyring class."""

    def setUp(self):
        self.gpg = TempKeyring()
        self.assertIn('homedir', self.gpg.context.options)

    def tearDown(self):
        del self.gpg

class TestKeyringBase(unittest.TestCase):
    """Base class for Keyring class tests.

    This shouldn't implement any tests that we don't want to see
    implemented every time.
    """

    def setUp(self):
        """setup the test environment

        we test using a temporary keyring because it's too dangerous
        otherwise.

        we are not using the TempKeyring class however, because we may
        want to keep that data for examination later. see the
        tearDown() function for that.
        """
        self.tmp = tempfile.mkdtemp(prefix="pygpg-")
        self.gpg = Keyring(self.tmp)
        self.assertEqual(self.gpg.context.options['homedir'], self.tmp)

    def tearDown(self):
        """trash the temporary directory we created"""
        shutil.rmtree(self.tmp, ignore_errors=True)

class TestKeyringBasics(TestKeyringBase):
    """Test the Keyring class base functionality."""

    def test_home(self):
        """test if the homedir is properly set and populated"""
        self.gpg.export_data('') # dummy call to make gpg populate his directory
        self.assertTrue(os.path.exists(self.tmp + '/pubring.gpg') or
                        os.path.exists(self.tmp + '/pubring.kbx'))

    def test_import(self):
        """make sure import_data returns true on known good data

        it should throw an exception if there's something wrong with the backend too
        """
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))

    def test_import_fail(self):
        """test that import_data() throws an error on wrong data"""
        self.assertFalse(self.gpg.import_data(''))

    def test_export(self):
        """test that we can export data similar to what we import

        @todo this will probably fail if tests are ran on a different GPG version
        """
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))
        k1 = re.sub(r'Version:.*$', r'', open(find_test_file('96F47C6A.asc')).read(), flags=re.MULTILINE)
        self.gpg.context.set_option('armor')
        self.gpg.context.set_option('export-options', 'export-minimal')
        self.gpg.context.set_option('no-emit-version')
        k2 = re.sub(r'Version:.*$', r'', self.gpg.export_data('96F47C6A'), flags=re.MULTILINE)
        self.assertEqual(k1,k2)

    def test_get_missing_secret_keys(self):
        """make sure we fail to get secret keys when they are missing"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('7B75921E.asc')).read()))
        # this shouldn't show anything, as this is just a public key blob
        self.assertFalse(self.gpg.get_keys('8DC901CE64146C048AD50FBB792152527B75921E', True, False))

    def test_export_secret(self):
        """make sure we can import and export secret data"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))
        self.secret = self.gpg.export_data('96F47C6A', True)
        self.assertTrue(self.secret)

    def test_list_imported_secrets(self):
        """make sure we can print imported secrets"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))
        self.assertTrue(self.gpg.get_keys(None, True, False))

    def test_empty_keyring(self):
        """a test should work on an empty keyring

        this is also a test of exporting an empty keyring"""
        self.assertEqual(self.gpg.export_data(), '')

    def test_sign_key_missing_key(self):
        """try to sign a missing key

        this should fail because we don't have the public key material
        for the requested key

        however, gpg returns the wrong exit code here, so we end up at
        looking if there is really no output
        """
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))
        with self.assertRaises(GpgRuntimeError):
            self.gpg.sign_key('7B75921E')

    def test_failed_revoke(self):
        self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read())
        self.gpg.import_data(open(find_test_file('96F47C6A-revoke.asc')).read())
        self.gpg.import_data(open(find_test_file('7B75921E.asc')).read())
        with self.assertRaises(GpgRuntimeError):
            self.gpg.sign_key('7B75921E', True)


class TestKeyringWithKeys(TestKeyringBase):
    def setUp(self):
        TestKeyringBase.setUp(self)
        self.assertTrue(self.gpg.import_data(open(find_test_file('7B75921E.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))

    def test_get_keys(self):
        """test that we can list the keys after importing them

        @todo we should check the data structure
        """
        # just a cute display for now
        for fpr, key in self.gpg.get_keys('96F47C6A').iteritems():
            print key

    def test_sign_key_wrong_user(self):
        """make sure sign_key with a erroneous local-user fails

        that is, even if all other conditions are ok"""
        self.gpg.context.set_option('local-user', '0000000F')
        with self.assertRaises(GpgRuntimeError):
            self.gpg.sign_key('7B75921E', True)

    @skipIfDatePassed('2017-06-01T00:00:00UTC')
    def test_sign_key_all_uids(self):
        """test signature of all uids of a key"""
        self.assertTrue(self.gpg.sign_key('7B75921E', True))
        self.gpg.context.call_command(['list-sigs', '7B75921E'])
        self.assertRegexpMatches(self.gpg.context.stdout, 'sig:::1:86E4E70A96F47C6A:[^:]*::::Second Test Key <unittests@monkeysphere.info>:10x:')

    @skipIfDatePassed('2017-06-01T00:00:00UTC')
    def test_sign_key_single_uid(self):
        """test signing a key with a single uid"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('323F39BD.asc')).read()))
        self.assertTrue(self.gpg.sign_key('323F39BD', True))
        self.gpg.context.call_command(['list-sigs', '323F39BD'])
        self.assertRegexpMatches(self.gpg.context.stdout, 'sig:::1:A31E75E4323F39BD:[^:]*::::Monkeysphere second test key <bar@example.com>:[0-9]*x:')

    @skipUnlessUnicodeLocale()
    @skipIfDatePassed('2017-06-01T00:00:00UTC')
    def test_sign_key_one_uid(self):
        """test signature of a single uid"""
        self.assertTrue(self.gpg.sign_key('Antoine Beaupré <anarcat@debian.org>'))
        self.gpg.context.call_command(['list-sigs', '7B75921E'])
        self.assertRegexpMatches(self.gpg.context.stdout, 'sig:::1:86E4E70A96F47C6A:[^:]*::::Second Test Key <unittests@monkeysphere.info>:10x:')

    @skipIfDatePassed('2017-06-01T00:00:00UTC')
    def test_sign_key_as_user(self):
        """normal signature with a signing user specified"""
        self.gpg.context.set_option('local-user', '96F47C6A')
        self.assertTrue(self.gpg.sign_key('7B75921E', True))

    @skipUnlessUnicodeLocale()
    @skipIfDatePassed('2017-06-01T00:00:00UTC')
    def test_sign_already_signed(self):
        """test if signing a already signed key fails with a meaningful message"""
        self.assertTrue(self.gpg.sign_key('Antoine Beaupré <anarcat@debian.org>'))
        with self.assertRaises(GpgRuntimeError) as e:
            self.gpg.sign_key('Antoine Beaupré <anarcat@debian.org>')
            self.assertIn('you already signed that key', str(e))

    def test_encrypt_decrypt_data_armored_untrusted(self):
        """test if we can encrypt data to our private key (and decrypt it)"""
        plaintext = 'i come in peace'

        # we trust all keys blindly to avoid having to set trust on that key
        self.gpg.context.set_option('trust-model', 'always')
        self.gpg.context.set_option('armor')
        cyphertext = self.gpg.encrypt_data(plaintext, '96F47C6A')
        self.assertTrue(cyphertext)

        p = self.gpg.decrypt_data(cyphertext)
        self.assertTrue(p)
        self.assertEqual(p, plaintext)

    def test_gen_key(self):
        """test key generation

        not implemented"""
        #self.fpr = self.gpg.gen_key()
        #self.assertTrue(self.fpr)
        pass

    def test_multi_secrets(self):
        """test if we get confused with multiple secret keys"""

        self.assertTrue(self.gpg.import_data(open(find_test_file('323F39BD.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('323F39BD-secret.asc')).read()))

        keys = self.gpg.get_keys(None, True, False)
        self.assertEqual(len(keys.keys()), 2)
        #for fpr, key in keys.iteritems():
        #    print >>sys.stderr, "key:", key

    def test_del_uid(self):
        """test uid deletion, gpg.del_uid()"""
        userid = 'Antoine Beaupré <anarcat@orangeseeds.org>'
        self.assertTrue(self.gpg.import_data(open(find_test_file('7B75921E.asc')).read()))
        found = False
        keys = self.gpg.get_keys('7B75921E')
        for fpr, key in keys.iteritems():
            for u, uid in key.uids.iteritems():
                self.assertIsInstance(uid, OpenPGPuid)
                if userid == uid.uid:
                    found = True
                    break
        self.assertTrue(found, "that we can find the userid before removing it")
        self.assertTrue(self.gpg.del_uid(fpr, userid))
        for fpr, key in self.gpg.get_keys('7B75921E').iteritems():
            for u, uid in key.uids.iteritems():
                self.assertNotEqual(userid, uid.uid)

    def test_del_uid_except(self):
        """see if we can easily delete all uids except a certain one"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('7B75921E.asc')).read()))
        userid = 'Antoine Beaupré <anarcat@orangeseeds.org>'
        keys = self.gpg.get_keys('7B75921E')
        todelete = []
        # XXX: otherwise test fails with GpgProtocolError: [Errno 0] expected "^\[GNUPG:\] GET_LINE keyedit.prompt", found "gpg: vérification de la base de confiance"
        self.gpg.context.set_option('trust-model', 'always')
        for fpr, key in keys.iteritems():
            for u, uid in key.uids.iteritems():
                if userid != uid.uid:
                    todelete.append(uid.uid)
            for uid in todelete:
                self.gpg.del_uid(fpr, uid)
        for fpr, key in self.gpg.get_keys('7B75921E').iteritems():
            for u, uid in key.uids.iteritems():
                self.assertEqual(userid, uid.uid)

    def test_verify_file(self):
        """test verify_file()"""
        self.assertTrue(self.gpg.verify_file(find_test_file('testfile.txt.asc'), find_test_file('testfile.txt')))

    def test_sign_revoked_uid(self):
        self.assertTrue(self.gpg.import_data(open(find_test_file('323F39BD.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('323F39BD-secret.asc')).read()))
        self.gpg.import_data(open(find_test_file('96F47C6A-revuid.asc')).read())
        # designate that new key as the signing key
        self.gpg.context.set_option('local-user', '323F39BD')
        self.assertTrue(self.gpg.sign_key('3F94240C918E63590B04152E86E4E70A96F47C6A', True))

        self.gpg.context.call_command(['list-sigs', '96F47C6A'])
        # try to find a revoked uid that is signed
        for uid in self.gpg.context.stdout.split('uid:'):
            if 'rev:' in uid:
                self.assertNotIn('sig:::1:A31E75E4323F39BD', uid)


class TestKeyringWithAbnormalKeys(TestKeyringBase):
    """this tests specifically weird keys"""
    def setUp(self):
        TestKeyringBase.setUp(self)
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A.asc')).read()))
        self.assertTrue(self.gpg.import_data(open(find_test_file('96F47C6A-secret.asc')).read()))

    @skipIfDatePassed('2018-02-03T00:00:00UTC')
    def test_wrongly_place_sigs(self):
        """test zack's key

        this provokes an exception with:

        monkeysign.gpg.GpgProtocolError: [Errno 0] expected
        "^\[GNUPG:\] GOT_IT", found "gpg: moving a key signature to
        the correct place"

        seems like it's a key not respecting standard:
        http://lists.nongnu.org/archive/html/sks-devel/2012-07/msg00122.html

        see https://bugs.debian.org/736120
        """
        self.assertTrue(self.gpg.import_data(open(find_test_file('6D866396.asc')).read()))
        self.assertTrue(self.gpg.sign_key('6D866396', True))
        self.gpg.context.call_command(['list-sigs', '6D866396'])
        self.assertRegexpMatches(self.gpg.context.stdout, 'sig:::1:86E4E70A96F47C6A:[^:]*::::Second Test Key <unittests@monkeysphere.info>:10x:')

    def test_broken_encoding(self):
        """test some key that has a non-standard encoding

        RFC4880 specifies that UIDs should be UTF-8, but someone this
        one isn't.

        see https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=736629
        """
        self.assertTrue(self.gpg.import_data(open(find_test_file('ECAA37C45C7E48CE.asc')).read()))
        key = self.gpg.get_keys('095D9EC8C995AB203DC260FEECAA37C45C7E48CE')
        key['095D9EC8C995AB203DC260FEECAA37C45C7E48CE'].__str__().encode('utf-8')

    def test_expired_subkeys(self):
        """test a key that has an expired subkey"""
        self.assertTrue(self.gpg.import_data(open(find_test_file('576407A629299233.asc')).read()))
        self.assertTrue(self.gpg.sign_key('576407A629299233', True))
        self.gpg.context.call_command(['list-sigs', '576407A629299233'])
        self.assertRegexpMatches(self.gpg.context.stdout, 'sig:::1:86E4E70A96F47C6A:[^:]*::::Second Test Key <unittests@monkeysphere.info>:10x:')


class TestOpenPGPkey(unittest.TestCase):
    def setUp(self):
        self.key = OpenPGPkey("""tru::1:1343350431:0:3:1:5
pub:-:1024:1:86E4E70A96F47C6A:1342795252:::-:::scESC:
fpr:::::::::3F94240C918E63590B04152E86E4E70A96F47C6A:
uid:-::::1342795252::214CB0EDA28F3CA8754A4D43B7CDB7B114171B3C::Test Key <foo@example.com>:
sub:-:1024:1:894EE34814B46386:1342795252::::::e:""")

    def test_no_dupe_uids(self):
        key = OpenPGPkey()
        self.assertEqual(key.uids, {})

    def test_format_fpr(self):
        expected = '3F94 240C 918E 6359 0B04  152E 86E4 E70A 96F4 7C6A'
        actual = self.key.format_fpr()
        self.assertEqual(expected, actual)

    def test_get_trust(self):
        self.assertEqual('unknown', self.key.get_trust())

class TestSecretOpenPGPkey(unittest.TestCase):
    def setUp(self):
        self.key = OpenPGPkey("""sec::1024:17:586073B34023702F:1110320887:1268438180:::::::::
fpr:::::::::C9E1F1230DBE47D57BAB3C60586073B34023702F:
uid:::::::2451063FCBB4D262938687C2D8F6B949B0A3AF01::The Anarcat <anarcat@anarcat.ath.cx>:
ssb::2048:16:C016FF12EB8D47BB:1110320966::::::::::""")

    def test_print(self):
        print self.key

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

"""This deliberately incomplete MUA will emulate behaviors of the
worst MUA available we can think of. It will behave like xdg-email
with extra bugs. For example:

1. parameters are only passed on the commandline
2. complete MIME messages will not work
3. it will return before processing the message
4. it will sleep for 5 seconds before actually "sending" the message

The message is appended to a temporary file in a mbox-like format
(ie. messages are just concatenated). The path to the file is printed
to stdout.

Return code should be 0 if all goes well or non-zero for any other
failure. Errors are logged to stderr.
"""

from __future__ import division, absolute_import
from __future__ import print_function, unicode_literals

import argparse
from email import Charset
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
# Python 2 backwards-compatible open
from io import open
import logging
import os
import tempfile
import time
import sys

logging.basicConfig(format='%(message)s', level=logging.DEBUG)


def parse_args():
    parser = argparse.ArgumentParser(description='Fake Mail User Agent',
                                     epilog=__doc__)
    parser.add_argument('--to', required=True,
                        help='email address to send the message to')
    parser.add_argument('--body', help='body of the message')
    parser.add_argument('--subject', help='body of the message')
    parser.add_argument('--attach', help='file to attach to the message')
    parser.add_argument('--sleep', default=5, type=int,
                        help='how long to sleep for (default: %(default)s)')
    parser.add_argument('--output', default=None,
                        help='where to write the messages (default: tmpfil)')
    return parser.parse_args()


def make_message(args):
    """create mime message with the given arguments"""

    # Override python's weird assumption that utf-8 text should be
    # encoded with base64, and instead use quoted-printable (for both
    # subject and body).  I can't figure out a way to specify QP
    # (quoted-printable) instead of base64 in a way that doesn't
    # modify global state. :-( see
    # http://radix.twistedmatrix.com/2010/07/how-to-send-good-unicode-email-with.html
    Charset.add_charset('utf-8', Charset.QP, Charset.QP, 'utf-8')

    msg = body = MIMEText(args.body, 'plain', 'utf-8')
    if args.attach:
        files = MIMEBase('text', 'plain', filename=args.attach)
        files.add_header('Content-Disposition', 'inline', filename='msg.asc')
        files.add_header('Content-Transfer-Encoding', '7bit')
        with open(args.attach, encoding='utf-8') as attach:
            files.set_payload(attach.read())

        msg = MIMEMultipart('mixed', None, (body, files))
    try:
        msg['Subject'] = Header(args.subject.encode('ascii'))
    except UnicodeDecodeError:
        msg['Subject'] = Header(args.subject, 'UTF-8')
    msg['From'] = 'unittests@localhost'
    msg['To'] = args.to
    return msg


def main():
    args = parse_args()
    if args.output is None:
        fd, tmpfile = tempfile.mkstemp()
        fd = open(fd, mode='a', encoding='utf-8')
        logging.debug('created tmpfile %s', tmpfile)
    elif args.output == '-':
        fd = sys.stdout
        tmpfile = '/dev/stdout'
    else:
        tmpfile = args.output
        fd = open(tmpfile, mode='w', encoding='utf-8')

    logging.debug('backgrounding')
    if os.fork():  # parent
        print(tmpfile)
        sys.exit(0)
    logging.debug('sleeping %d seconds', args.sleep)
    time.sleep(args.sleep)
    logging.debug('generating MIME message')
    msg = make_message(args)
    logging.debug('writing to file')
    fd.write(msg.as_string().decode('utf-8'))
    fd.close()

if __name__ == '__main__':
    main()



# bug-triage -- bug triage and forward tool.
# Copyright (C) 2007  Gustavo R. Montesino
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# This file was imported from the bug-triage tool after reading this blog post:
# http://grmontesino.blogspot.ca/2007/07/exception-handling-on-pygtk.html
# Other solutions based on sys.excepthook may work better, see:
# http://faq.pygtk.org/index.py?req=show&file=faq20.010.htp
# both links are broken above, see instead:
# http://www.sysfs.be/downloads/gtkexcepthook.py
# https://github.com/mypaint/mypaint/blob/master/gui/gtkexcepthook.py
# the latter also has hooks to send github bug reports!

import pygtk
pygtk.require("2.0")
import gtk
import traceback

import monkeysign.ui
import monkeysign.translation

def msg_exception(exception):
    """Shows information about a exception in a gtk dialog"""

    msg = ExceptionDialog(exception)
    msg.run()
    msg.destroy()

def errorhandler(f):
    """Error handler decorator"""
    def wrapper(*args, **kargs):
        try:
            return f(*args, **kargs)
        except Exception as instance:
            if type(instance) not in [KeyboardInterrupt, SystemExit]:
                msg_exception(instance)
    return wrapper

class ExceptionDialog(gtk.MessageDialog):
    def __init__(self, instance):
        gtk.MessageDialog.__init__(self, buttons=gtk.BUTTONS_CLOSE, type=gtk.MESSAGE_ERROR)
        self.set_resizable(True)
        self.set_markup(_("An error has occured:\n%r\nYou should save "
                "your work and restart the application. If the error "
                "occurs again please report it to the developer." % str(instance)))
        self.set_title(_("Error"))
        expander = gtk.Expander(_("Exception Details"))
        self.vbox.pack_start(expander)
        textview = gtk.TextView()
        text = "\n".join(monkeysign.gtkui.MonkeysignScanUi.sysinfo()) \
               + "\n" + traceback.format_exc()
        textview.get_buffer().set_text(text)
        expander.add(self.scrolled(textview))
        self.show_all()

    def scrolled(self, widget, shadow=gtk.SHADOW_NONE):
        window = gtk.ScrolledWindow()
        window.set_shadow_type(shadow)
        window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        if widget.set_scroll_adjustments(window.get_hadjustment(),
                                          window.get_vadjustment()):
            window.add(widget)
        else:
            window.add_with_viewport(widget)
        return window

class test:
    @errorhandler
    def __init__(self):
        raise Exception("test exception")

if __name__ == '__main__':
    test()
            
# vim: tabstop=4 expandtab shiftwidth=4     



# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
import os
import sys
import msgfmt
from distutils.command.build import build
from distutils.core import Command
import locale
import gettext
import pkg_resources

# Initialize gettext, taken from deluge 1.3.3 (GPL3)
try:
    locale.setlocale(locale.LC_ALL, '')
    if hasattr(locale, "bindtextdomain"):
        locale.bindtextdomain('monkeysign', pkg_resources.resource_filename('monkeysign', "po"))
    if hasattr(locale, "textdomain"):
        locale.textdomain('monkeysign')
    gettext.install('monkeysign', pkg_resources.resource_filename('monkeysign', "po"), unicode=True, names='ngettext')
except Exception as e:
    print ("Unable to initialize translations: %s" % e)
    import __builtin__
    __builtin__.__dict__["_"] = lambda x: x

# stolen from deluge-1.3.3 (GPL3)
class build_trans(Command):
    description = 'Compile .po files into .mo files'

    user_options = [
            ('build-lib=', None, "lib build folder"),
            ('po-dir=', 'p', 'directory where .po files are stored, relative to the current directory'),
    ]

    def initialize_options(self):
        self.build_lib = None
        self.po_dir = 'po/'

    def finalize_options(self):
        self.set_undefined_options('build', ('build_lib', 'build_lib'))

    def run(self):
        po_dir = self.po_dir

        appname = self.distribution.get_name()
        self.announce('compiling po files from %s' % po_dir, 2)
        uptoDate = False
        for path, names, filenames in os.walk(po_dir):
            for f in filenames:
                uptoDate = False
                if f.endswith('.po'):
                    lang = f[:len(f) - 3]
                    src = os.path.join(path, f)
                    dest_path = os.path.join(self.build_lib, appname, 'po', lang, \
                        'LC_MESSAGES')
                    dest = os.path.join(dest_path, appname + '.mo')
                    if not os.path.exists(dest_path):
                        os.makedirs(dest_path)
                    if not os.path.exists(dest):
                        sys.stdout.write('%s, ' % lang)
                        sys.stdout.flush()
                        msgfmt.make(src, dest)
                    else:
                        src_mtime = os.stat(src)[8]
                        dest_mtime = os.stat(dest)[8]
                        if src_mtime > dest_mtime:
                            sys.stdout.write('%s, ' % lang)
                            sys.stdout.flush()
                            msgfmt.make(src, dest)
                        else:
                            uptoDate = True
                            
        if uptoDate:
            self.announce('po files already upto date.', 2)
        else:
            self.announce('done', 2)


build.sub_commands.append(('build_trans', None))



#!/usr/bin/env python

import sys

try:
    from gi.repository import Gtk
except:
    print "You need to install pyGTK or GTKv2 ",
    sys.exit(1)

class Handler:
    """
    This handler gets called by Gtk.Builder() based on the signals
    defined in the XML UI.
    """
    def __init__(self, builder):
        self.builder = builder

    def quit(self, *args):
        Gtk.main_quit(*args)

    def enablecamera(self, button):
        button.set_label('Disable camera')
        button.disconnect_by_func(self.enablecamera)
        button.set_image(self.builder.get_object('previous-image'))
        button.connect('clicked', self.disablecamera)

    def disablecamera(self, button):
        button.set_label('Enable camera')
        button.disconnect_by_func(self.disablecamera)
        button.set_image(self.builder.get_object('next-image'))
        button.connect('clicked', self.enablecamera)

    def button(self, button):
        print "Hello World!"
        print "I am a button!" + repr(button)

class monkeysharegui:
    """
    This class builds the UI from the glade files and shows all
    windows, but doesn't start the GTK threads.
    """
    def __init__(self):
        """
        """
        builder = Gtk.Builder()
        builder.add_from_file(sys.argv[1])
        builder.connect_signals(Handler(builder))
        window = builder.get_object(sys.argv[2])
        window.show_all()
        return

try:
    monkeysharegui()
except IndexError:
    print "pass the name of the glade file to load and the name of the main window"
    sys.exit(1)
except AttributeError:
    print "wrong window name"
    sys.exit(1)
Gtk.main()



# -*- coding: utf-8 -*-

"""build_manpage command -- Generate man page from setup()"""

import os
import datetime
import time
from distutils.command.build import build
from distutils.core import Command
from distutils.errors import DistutilsOptionError
import subprocess  # nosec


class build_slides(Command):

    description = 'Generate the HTML presentation with rst2s5.'

    user_options = [
        ('file=', 'f', 'rst file'),
        ]

    def initialize_options(self):
        self.file = None

    def finalize_options(self):
        if self.file is None:
            raise DistutilsOptionError('\'file\' option is required')

    def run(self):
        html = os.path.join(os.path.dirname(self.file), os.path.splitext(os.path.basename(self.file))[0] + '.html')
        self.announce('processing slides from %s to %s' % (self.file, html), 2)
        subprocess.call(['rst2s5', '--theme', 'default', self.file, html])  # nosec

def has_rst2s5(build):
    """predicate for this class: do not fail if rst2s5 is missing"""
    return (os.system('rst2s5 < /dev/null > /dev/null') == 0)  # nosec

# (function, predicate), see http://docs.python.org/2/distutils/apiref.html#distutils.cmd.Command.sub_commands
build.sub_commands.append(('build_slides', has_rst2s5))
build.sub_commands.append(('build_sphinx', None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Library of test tools that can be reused across different test suites.
"""

import locale
import os.path
import pkg_resources
import signal
import sys
import time
import unittest

class AlarmException(IOError):
    pass

def handle_alarm(signum, frame):
    raise AlarmException('timeout in %s' % frame.f_code.co_name)

class TestTimeLimit(unittest.TestCase):
    """a test with a timeout"""

    # in seconds
    timeout = 3

    def setUp(self):
        signal.signal(signal.SIGALRM, handle_alarm)
        signal.setitimer(signal.ITIMER_REAL, self.timeout)

    def tearDown(self):
        signal.alarm(0)

class TestTimeLimitSelfTest(TestTimeLimit):

    # 10ms
    timeout = 0.01

    def test_signal(self):
        with self.assertRaises(AlarmException):
            time.sleep(1)


def find_test_file(name):
    try:
        pkg = pkg_resources.Requirement.parse("monkeysign")
        path = os.path.join('monkeysign', 'tests', 'files', name)
        return pkg_resources.resource_filename(pkg, path)
    except pkg_resources.DistributionNotFound:
        return os.path.join(os.path.dirname(__file__), 'files', name)


class TestTestFile(unittest.TestCase):
    def test_test_file(self):
        p = find_test_file('testfile.txt')
        self.assertTrue(os.path.exists(p))


def skipUnlessUnicodeLocale():
    try:
        u"é".encode(sys.stdout.encoding or locale.getpreferredencoding(True))
    except UnicodeEncodeError:
        return unittest.skip('test requires unicode locale')
    else:
        return lambda func: func


def skipIfDatePassed(date, fmt='%Y-%m-%dT%H:%M:%S%Z'):
    # date is in ISO-8601 format by default
    # XXX: should be auto-generated from the key material, or we
    # should be able to use gpg --faked-system-time but that doesn't
    # work in 1.x
    if time.gmtime() >= time.strptime(date, fmt):
        return unittest.skip('date %s is in the past' % date)
    else:
        return lambda func: func


def skipUnlessNetwork():
    """add a knob to disable network tests

    to disable network tests, use PYTEST_USENETWORK=no. by default, it
    is assumed there is network access.

    this is mainly to deal with Debian packages that are built in
    network-less chroots. unfortunately, there is no standard
    environment in dpkg-buildpackage or ./debian/rules binary that we
    can rely on to disable tests, so we revert to a custom variable
    that can hopefully make it up to the pybuild toolchain

    I looked at DEB_BUILD_OPTIONS=network, but that is not standard
    and only mentionned once here:

    https://lists.debian.org/debian-devel/2009/09/msg00992.html

    DEB_BUILD_OPTIONS is also not set by default, so it's not a good
    way to detect Debian package builds

    pbuilder uses USENETWORK=no/yes, schroot uses UNSHARE_NET, but
    those are not standard either, see:

    https://github.com/spotify/dh-virtualenv/issues/77
    https://github.com/codelibre-net/schroot/blob/2e3d015a759d2b5106e851af34c8d5974d84f18e/lib/schroot/chroot/facet/unshare.cc
    """

    if ('PYTEST_USENETWORK' in os.environ
       and 'no' in os.environ.get('PYTEST_USENETWORK', '')):
        return unittest.skip('network tests disabled (PYTEST_USENETWORK=no)')
    else:
        return lambda func: func


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#    Copyright (C) 2012-2013 Antoine Beaupré <anarcat@orangeseeds.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from glob import glob
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import monkeysign
import monkeysign.documentation
import monkeysign.build_manpage
import monkeysign.translation

setup(name='monkeysign',
      description=monkeysign.__description__,
      long_description=monkeysign.__long_description__,
      use_scm_version={
          'write_to': 'monkeysign/_version.py',
      },
      setup_requires=['sphinx', 'setuptools_scm'],
      install_requires=['pysocks'],
      author='Antoine Beaupré',
      author_email='anarcat@debian.org',
      url=monkeysign.__website__,
      packages=['monkeysign', 'monkeysign.tests'],
      package_data={
          'monkeysign.tests': ['files/*'],
      },
      scripts=['scripts/monkeysign', 'scripts/monkeyscan'],
      cmdclass={'build_manpage': monkeysign.build_manpage.BuildManPage,
                'build_trans': monkeysign.translation.build_trans,
                'build_slides': monkeysign.documentation.build_slides,
                },
      data_files=[('share/man/man1', glob('man/*.1')),
                  ('share/pixmaps', ['doc/ui-mockups/logo.simple.png']),
                  ('share/icons/hicolor/scalable',
                   ['doc/ui-mockups/logo.simple.svg']),
                  ],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Information Technology',
          'Intended Audience :: Legal Industry',
          'Intended Audience :: Telecommunications Industry',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.4',
          'Programming Language :: Python :: 2.5',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Operating System :: OS Independent',
          'Environment :: X11 Applications :: GTK',
          'Environment :: Console',
          'Natural Language :: English',
          'Natural Language :: French',
          'Natural Language :: Italian',
          'Natural Language :: Spanish',
          'Topic :: Communications :: Email',
          'Topic :: Multimedia :: Video :: Capture',
          'Topic :: Security :: Cryptography',
          'Topic :: Software Development :: Libraries :: Python Modules'
          ],
      test_suite='monkeysign.tests',
      )
